# -*- coding: utf_8 -*-
"""Dynamic Analyzer Operations."""
import json
import logging
import os
import random
import shlex
import subprocess
import threading
from pathlib import Path

from django.conf import settings
from django.views.decorators.http import require_http_methods

from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.security import is_attack_pattern
from mobsf.DynamicAnalyzer.views.android.environment import (
    Environment,
)
from mobsf.MobSF.security import cmd_injection_check
from mobsf.MobSF.utils import (
    docker_translate_localhost,
    get_adb,
    get_device,
    is_md5,
    is_number,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerAndroid
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)


# Helpers

def get_package_name(checksum):
    """Get Package Name from DB or Device."""
    try:
        static_android_db = StaticAnalyzerAndroid.objects.get(
            MD5=checksum)
        return static_android_db.PACKAGE_NAME
    except Exception:
        pkg_file = Path(settings.DWD_DIR) / 'packages.json'
        if not pkg_file.exists():
            return None
        with pkg_file.open(encoding='utf-8') as src:
            packages = json.load(src)
        if packages.get(checksum):
            return packages[checksum][0]
        return None
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def mobsfy(request, api=False):
    """Configure Instance for Dynamic Analysis."""
    logger.info('MobSFying Android instance')
    data = {}
    msg = 'Connection failed'
    try:
        identifier = request.POST['identifier']
        if cmd_injection_check(identifier):
            # Additional Check, not required
            data = {
                'status': 'failed',
                'message': 'Command Injection Detected',
            }
            return send_response(data, api)
        identifier = docker_translate_localhost(identifier)
        create_env = Environment(identifier)
        if not create_env.connect_n_mount():
            data = {'status': 'failed', 'message': msg}
            return send_response(data, api)
        version = create_env.mobsfy_init()
        if not version:
            data = {'status': 'failed', 'message': msg}
            return send_response(data, api)
        else:
            data = {'status': 'ok', 'android_version': version}
    except Exception as exp:
        logger.exception('MobSFying Android instance failed')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def execute_adb(request, api=False):
    """Execute ADB Commands."""
    data = {'status': 'ok', 'message': ''}
    cmd = request.POST['cmd']
    if cmd:
        args = [get_adb(),
                '-s',
                get_device()]
        try:
            proc = subprocess.Popen(
                args + cmd.split(' '),  # lgtm [py/command-line-injection]
                stdout=subprocess.PIPE,  # Expected, cmd execute inside VM/AVD
                stderr=subprocess.PIPE)
            stdout, stderr = proc.communicate()
        except Exception:
            logger.exception('Executing ADB Commands')
        if stdout or stderr:
            out = stdout or stderr
            out = out.decode('utf8', 'ignore')
        else:
            out = ''
        data = {'status': 'ok', 'message': out}
    return send_response(data, api)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def get_component(request):
    """Get Android Component."""
    data = {}
    try:
        env = Environment()
        comp = request.POST['component']
        bin_hash = request.POST['hash']
        if is_attack_pattern(comp) or not is_md5(bin_hash):
            return invalid_params()
        comp = env.android_component(bin_hash, comp)
        data = {'status': 'ok', 'message': comp}
    except Exception as exp:
        logger.exception('Getting Android Component')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def run_apk(request):
    """Run Android APK."""
    data = {}
    try:
        env = Environment()
        md5_hash = request.POST['hash']
        if not is_md5(md5_hash):
            return invalid_params()
        pkg = get_package_name(md5_hash)
        if not pkg:
            return invalid_params()
        env.run_app(pkg)
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Running the App')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def take_screenshot(request, api=False):
    """Take Screenshot."""
    logger.info('Taking screenshot')
    data = {}
    try:
        env = Environment()
        bin_hash = request.POST['hash']
        if not is_md5(bin_hash):
            return invalid_params(api)
        data = {}
        rand_int = random.randint(1, 1000000)
        screen_dir = os.path.join(settings.UPLD_DIR,
                                  bin_hash + '/screenshots-apk/')
        if not os.path.exists(screen_dir):
            os.makedirs(screen_dir)
        outile = '{}screenshot-{}.png'.format(
            screen_dir,
            str(rand_int))
        env.screen_shot(outile)
        logger.info('Screenshot captured')
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Taking screenshot')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def screen_cast(request):
    """ScreenCast."""
    data = {
        'status': 'failed',
        'message': 'Failed to stream screen'}
    try:
        env = Environment()
        b64dat = env.screen_stream()
        data = {
            'status': 'ok',
            'message': f'data:image/png;base64,{b64dat}'}
    except Exception as exp:
        logger.exception('Screen streaming')
        data['message'] = str(exp)
    return send_response(data)
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def touch(request):
    """Sending Touch/Swipe/Text Events."""
    data = {
        'status': 'failed',
        'message': '',
    }
    try:
        env = Environment()
        x = request.POST['x']
        y = request.POST['y']
        event = request.POST['event']
        max_x = request.POST.get('max_x', 0)
        max_y = request.POST.get('max_y', 0)

        if event == 'text':
            args = ['text', shlex.quote(x)]
        else:
            if (not is_number(x)
                    or not is_number(y)
                    or not is_number(max_x)
                    or not is_number(max_y)):
                return data
            # Should not be greater than max screen size
            swipe_x = str(min(int(float(x)) + 500, int(float(max_x))))
            swipe_y = str(min(int(float(y)) + 500, int(float(max_y))))

            if event == 'enter':
                args = ['keyevent', '66']
            elif event == 'backspace':
                args = ['keyevent', '67']
            elif event == 'left':
                args = ['keyevent', '21']
            elif event == 'right':
                args = ['keyevent', '22']
            elif event == 'swipe_up':
                args = ['swipe', x, y, x, swipe_y]
            elif event == 'swipe_down':
                args = ['swipe', x, swipe_y, x, y]
            elif event == 'swipe_left':
                args = ['swipe', x, y, swipe_x, y]
            elif event == 'swipe_right':
                args = ['swipe', swipe_x, y, x, y]
            else:
                args = ['tap', x, y]
        threading.Thread(target=env.adb_command,
                         args=(['input'] + args, True),
                         daemon=True).start()
        data = {'status': 'ok'}
    except Exception as exp:
        logger.exception('Sending Touchscreen Events')
        data['message'] = str(exp)
    return send_response(data)
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def mobsf_ca(request, api=False):
    """Install and Remove MobSF Proxy RootCA."""
    data = {}
    try:
        env = Environment()
        action = request.POST['action']
        if action == 'install':
            env.install_mobsf_ca(action)
            data = {'status': 'ok', 'message': 'installed'}
        elif action == 'remove':
            env.install_mobsf_ca(action)
            data = {'status': 'ok', 'message': 'removed'}
        else:
            data = {'status': 'failed',
                    'message': 'Action not supported'}
    except Exception as exp:
        logger.exception('MobSF RootCA Handler')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def global_proxy(request, api=False):
    """Set/unset global proxy."""
    data = {}
    try:
        env = Environment()
        version = env.get_android_version()
        action = request.POST['action']
        if action == 'set':
            env.set_global_proxy(version)
            data = {'status': 'ok', 'message': 'set'}
        elif action == 'unset':
            env.unset_global_proxy()
            data = {'status': 'ok', 'message': 'unset'}
        else:
            data = {'status': 'failed',
                    'message': 'Action not supported'}
    except Exception as exp:
        logger.exception('MobSF Global Proxy')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)



# -*- coding: utf_8 -*-
"""Handle XAPK File."""
import logging
import subprocess
from json import load
from shutil import move
from pathlib import Path

from django.conf import settings

from mobsf.StaticAnalyzer.views.common.shared_func import unzip
from mobsf.MobSF.security import is_safe_path
from mobsf.MobSF.utils import (
    append_scan_status,
    find_java_binary,
    is_file_exists,
)

logger = logging.getLogger(__name__)


def handle_xapk(app_dic):
    """Unzip and Extract APK."""
    data = None
    checksum = app_dic['md5']
    xapk = app_dic['app_dir'] / f'{checksum}.xapk'
    apk = app_dic['app_dir'] / f'{checksum}.apk'
    files = unzip(
        checksum,
        xapk.as_posix(),
        app_dic['app_dir'])
    if 'manifest.json' not in files:
        logger.error('Manifest file not found in XAPK')
        return False
    manifest = app_dic['app_dir'] / 'manifest.json'
    with open(manifest, encoding='utf8', errors='ignore') as f:
        data = load(f)
    if not data:
        logger.error('Manifest file is empty')
        return False
    apks = data.get('split_apks')
    if not apks:
        logger.error('Split APKs not found')
        return False
    for a in apks:
        if a['id'] == 'base':
            base_apk = app_dic['app_dir'] / a['file']
            if is_safe_path(app_dic['app_dir'], base_apk, a['file']):
                move(base_apk, apk)
                return True
    return None


def handle_split_apk(app_dic):
    """Unzip and Extract Split APKs."""
    checksum = app_dic['md5']
    apks = app_dic['app_dir'] / f'{checksum}.apk'
    # Check if previously extracted
    manifest = app_dic['app_dir'] / 'AndroidManifest.xml'
    if manifest.exists():
        return True
    for apk in unzip(checksum, apks.as_posix(), app_dic['app_dir']):
        full_path = app_dic['app_dir'] / apk
        safe_path = is_safe_path(app_dic['app_dir'], full_path, apk)
        if apk.endswith('base.apk') and safe_path:
            move(full_path, apks)
            return True
        if ('config.' not in apk.lower()
                and apk.endswith('.apk')
                and safe_path):
            move(full_path, apks)
            return True
    return None


def handle_aab(app_dic):
    """Convert AAB to APK using bundletool."""
    try:
        checksum = app_dic['md5']
        aab_path = app_dic['app_dir'] / f'{checksum}.aab'
        apks = aab_path.with_suffix('.apks')
        apk = aab_path.with_suffix('.apk')
        tools_dir = app_dic['tools_dir']
        # Check if previously converted
        manifest = app_dic['app_dir'] / 'AndroidManifest.xml'
        if manifest.exists():
            return True
        msg = 'Converting AAB to APK'
        logger.info(msg)
        append_scan_status(checksum, msg)
        if (getattr(settings, 'BUNDLE_TOOL', '')
                and len(settings.BUNDLE_TOOL) > 0
                and is_file_exists(settings.BUNDLE_TOOL)):
            bundletool = settings.BUNDLE_TOOL
        else:
            bundletool = Path(tools_dir) / 'bundletool-all-1.17.2.jar'
            bundletool = bundletool.as_posix()
        args = [
            find_java_binary(),
            '-jar',
            bundletool,
            'build-apks',
            f'--bundle={aab_path.as_posix()}',
            f'--output={apks.as_posix()}',
            '--mode=universal',
        ]
        if not apks.exists() and aab_path.exists():
            # Convert AAB to APKS
            subprocess.run(args, timeout=300)
        # Extract APK from APKS
        for apk_file in unzip(checksum, apks.as_posix(), app_dic['app_dir']):
            full_path = app_dic['app_dir'] / apk_file
            safe_path = is_safe_path(app_dic['app_dir'], full_path, apk_file)
            if apk_file == 'universal.apk' and safe_path:
                move(full_path, apk)
                apks.unlink()
                return True
        raise Exception('Unable to convert AAB to APK')
    except subprocess.TimeoutExpired as exp:
        msg = 'Converting AAB to APK timed out'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
    except Exception as exp:
        msg = 'Failed to convert AAB to APK'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return None



# -*- coding: utf_8 -*-
"""Security tests on data in transit."""
import re
import logging
from threading import Thread
from json import dump
from pathlib import Path

from django.conf import settings

from mobsf.DynamicAnalyzer.views.android.frida_core import (
    Frida,
)
from mobsf.DynamicAnalyzer.tools.webproxy import (
    get_http_tools_url,
    get_traffic,
    stop_httptools,
)


HTTPS = re.compile(r'(GET|POST|HEAD|PUT|DELETE|'
                   r'CONNECT|OPTIONS|TRACE|PATCH) https:\/\/')
HTTP = re.compile(r'(GET|POST|HEAD|PUT|DELETE|'
                  r'CONNECT|OPTIONS|TRACE|PATCH) http:\/\/')
logger = logging.getLogger(__name__)


def detect_traffic(data):
    """Detect Traffic."""
    status = {
        'http': False,
        'https': False,
    }
    if not data:
        return status
    if HTTPS.search(data):
        status['https'] = True
    if HTTP.search(data):
        status['http'] = True
    return status


def run_tls_tests(request, md5_hash, env, package, test_pkg, duration):
    """Run all TLS test cases."""
    test_status = {
        'tls_misconfigured': False,
        'no_tls_pin_or_transparency': False,
        'pin_or_transparency_bypassed': False,
        'has_cleartext': False,
    }
    version = env.get_android_version()
    env.enable_adb_reverse_tcp(version)
    env.set_global_proxy(version)
    # Test 1: Remove Root CA, Run App, No TLS Pinning Bypass
    env.adb_command(['am', 'force-stop', package], True)
    logger.info('Running TLS Misconfiguration Test')
    env.configure_proxy(test_pkg, request)
    env.install_mobsf_ca('remove')
    env.run_app(package)
    env.wait(duration)
    stop_httptools(get_http_tools_url(request))
    traffic = get_traffic(test_pkg)
    res = detect_traffic(traffic)
    if res['http']:
        test_status['has_cleartext'] = True
    if res['https']:
        test_status['tls_misconfigured'] = True
    # Test 2: Install Root CA, Run App, No TLS Pinning Bypass
    env.adb_command(['am', 'force-stop', package], True)
    logger.info('Running TLS Pinning/Certificate Transparency Test')
    env.configure_proxy(test_pkg, request)
    env.run_app(package)
    env.wait(duration)
    stop_httptools(get_http_tools_url(request))
    traffic = get_traffic(test_pkg)
    res = detect_traffic(traffic)
    if res['http']:
        test_status['has_cleartext'] = True
    if res['https']:
        test_status['no_tls_pin_or_transparency'] = True
    # Test 3: MobSF TLS Pinning Bypass Check
    env.adb_command(['am', 'force-stop', package], True)
    logger.info('Running TLS Pinning/Certificate Transparency Bypass Test')
    env.configure_proxy(test_pkg, request)
    frd = Frida(
        md5_hash,
        package,
        ['ssl_pinning_bypass', 'debugger_check_bypass', 'root_bypass'],
        None,
        None,
        None,
    )
    frd.spawn()
    Thread(target=frd.session, args=(None, None), daemon=True).start()
    env.wait(duration)
    stop_httptools(get_http_tools_url(request))
    traffic = get_traffic(test_pkg)
    res = detect_traffic(traffic)
    if res['http']:
        test_status['has_cleartext'] = True
    if res['https']:
        test_status['pin_or_transparency_bypassed'] = True
    env.adb_command(['am', 'force-stop', package], True)
    out = Path(settings.UPLD_DIR) / md5_hash / 'mobsf_tls_tests.json'
    with out.open('w', encoding='utf-8') as target:
        dump(test_status, target)
    return test_status



# -*- coding: utf_8 -*-
"""Corellium SSH.

Corellium SSH Utilities , modified for MobSF.
Supports SSH over Jump Host
Local Port Forward
Remote Port Forward
SSH Shell Exec
SFTP File Upload
SFTP File Download
"""
# Copyright (C) 2003-2007  Robey Pointer <robeypointer@gmail.com>
#
# This file is part of paramiko.
#
# Paramiko is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# Paramiko is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Paramiko; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
# Modified for MobSF.
import io
import logging
import select
import socket
import socketserver
from threading import Thread
from pathlib import Path

import paramiko

from django.conf import settings

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey


logger = logging.getLogger(__name__)


def generate_keypair_if_not_exists(location):
    """Generate RSA key pair."""
    prv = location / 'ssh_key.private'
    pub = location / 'ssh_key.public'
    if prv.exists() and pub.exists():
        # Keys Exists
        return prv.read_bytes(), pub.read_bytes()
    logger.info('Generating RSA key pair for Corellium SSH')

    # Generate private/public key pair
    private_key = Ed25519PrivateKey.generate()
    public_key = private_key.public_key()

    # OpenSSH friendly
    private_bytes = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.OpenSSH,
        encryption_algorithm=serialization.NoEncryption())

    public_bytes = public_key.public_bytes(
        encoding=serialization.Encoding.OpenSSH,
        format=serialization.PublicFormat.OpenSSH)
    prv.write_bytes(private_bytes)
    pub.write_bytes(public_bytes)
    return private_bytes, public_bytes


def parse_ssh_string(ssh):
    """Parse SSH connection string."""
    ssh_dict = {}
    sp = ssh.split(' ')
    bastion = sp[2]
    private = sp[3]
    ssh_dict['bastion_user'] = bastion.split('@')[0]
    ssh_dict['bastion_host'] = bastion.split('@')[1]
    ssh_dict['private_user'] = private.split('@')[0]
    ssh_dict['private_ip'] = private.split('@')[1]
    return ssh_dict


def sock_chan_handler(sock, chan):
    """Socket and Channel Handler."""
    try:
        while True:
            r, w, x = select.select([sock, chan], [], [])
            if sock in r:
                data = sock.recv(1024)
                if len(data) == 0:
                    break
                chan.send(data)
            if chan in r:
                data = chan.recv(1024)
                if len(data) == 0:
                    break
                sock.send(data)
    except ConnectionResetError:
        pass
    finally:
        if chan:
            chan.close()
        if sock:
            sock.close()


# Local Port Forward
class ForwardServer(socketserver.ThreadingTCPServer):
    daemon_threads = True
    allow_reuse_address = True


class Handler(socketserver.BaseRequestHandler):
    def handle(self):
        chan = None
        sock = self.request
        try:
            chan = self.ssh_transport.open_channel(
                'direct-tcpip',
                (self.chain_host, self.chain_port),
                sock.getpeername(),
            )
        except paramiko.SSHException:
            # SSH tunnel closed, try opening again
            ssh_jumphost_port_forward(self.ssh_string)
        except Exception as e:
            logger.info(
                'Incoming request to %s:%d failed: %s',
                self.chain_host, self.chain_port, repr(e))
            return
        if chan is None:
            logger.info(
                'Incoming request to %s:%d was rejected by the SSH server.',
                self.chain_host, self.chain_port)
            return

        logger.info(
            'Connected!  Tunnel open %r -> %r -> %r',
            sock.getpeername(),
            chan.getpeername(),
            (self.chain_host, self.chain_port))
        peername = sock.getpeername()
        sock_chan_handler(sock, chan)
        logger.info('Tunnel closed from %r', peername)


def forward_tunnel(local_port, remote_host, remote_port, transport, ssh):
    # this is a little convoluted, but lets me configure things for the Handler
    # object.  (SocketServer doesn't give Handlers any way to access the outer
    # server normally.)
    class SubHander(Handler):
        chain_host = remote_host
        chain_port = remote_port
        ssh_transport = transport
        ssh_string = ssh

    try:
        ForwardServer(('', local_port), SubHander).serve_forever()
    except OSError:
        logger.info('Port Forwarding Already in place')


# Remote Port Forward
def handler(chan, host, port):
    sock = socket.socket()
    try:
        sock.connect((host, port))
    except ConnectionRefusedError:
        # Proxy server is stopped
        return
    except Exception:
        logger.info('Forwarding request to %s:%d failed', host, port)
        return
    sock_chan_handler(sock, chan)


def reverse_forward_tunnel(server_port, remote_host, remote_port, transport):
    try:
        transport.request_port_forward('', server_port)
        while True:
            chan = transport.accept(1000)
            if chan is None:
                continue
            Thread(
                target=handler,
                args=(chan, remote_host, remote_port),
                daemon=True).start()
    except paramiko.SSHException as exp:
        if 'forwarding request denied' in str(exp):
            # Handle TCP forwarding request denied
            # Happens if already forwarding port
            pass
        else:
            logger.exception('SSH Remote Port Forward Exception')


def ssh_jump_host(ssh_string):
    """Connect to SSH over a bastion."""
    ssh_dict = parse_ssh_string(ssh_string)
    bastion_user = ssh_dict['bastion_user']
    bastion_host = ssh_dict['bastion_host']
    user = ssh_dict['private_user']
    private_ip = ssh_dict['private_ip']

    home = Path(settings.UPLD_DIR).parent
    generate_keypair_if_not_exists(home)
    keyf = home / 'ssh_key.private'
    jumpbox = paramiko.SSHClient()
    jumpbox.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    jumpbox.connect(
        bastion_host,
        username=bastion_user,
        key_filename=keyf.as_posix())

    jumpbox_transport = jumpbox.get_transport()
    src_addr = (private_ip, 22)
    dest_addr = (private_ip, 22)
    jumpbox_channel = jumpbox_transport.open_channel(
        'direct-tcpip', dest_addr, src_addr)

    target = paramiko.SSHClient()
    target.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    target.connect(
        private_ip,
        username=user,
        sock=jumpbox_channel,
        password='alpine')
    return target, jumpbox


def ssh_jumphost_port_forward(ssh_string):
    """SSH over Jump Host and Local Port Forward."""
    target, _jumpbox = ssh_jump_host(ssh_string)
    # Frida port
    forward_port = 27042
    remote_host = '127.0.0.1'
    forward_tunnel(
        forward_port,
        remote_host,
        forward_port,
        target.get_transport(),
        ssh_string)


def ssh_jumphost_reverse_port_forward(ssh_string):
    """SSH over Jump Host and Remote Port Forward."""
    target, _jumpbox = ssh_jump_host(ssh_string)
    # HTTPS proxy port
    port = settings.PROXY_PORT
    remote_host = '127.0.0.1'
    reverse_forward_tunnel(
        port,
        remote_host,
        port,
        target.get_transport(),
    )


def ssh_execute_cmd(target, cmd):
    """Execute SSH command."""
    _stdin, _stdout, _stderr = target.exec_command(cmd)
    stdout = _stdout.read().decode(encoding='utf-8', errors='ignore')
    stderr = _stderr.read().decode(encoding='utf-8', errors='ignore')
    return f'{stdout}\n{stderr}'


def ssh_file_upload(ssh_conn_string, fobject, fname):
    """File Upload over SFTP."""
    target, jumpbox = ssh_jump_host(ssh_conn_string)
    with target.open_sftp() as sftp:
        rfile = Path(fname.replace('..', '')).name
        sftp.putfo(fobject, f'/tmp/{rfile}')
    target.close()
    jumpbox.close()


def ssh_file_download(target, remote_path):
    """File Download over SFTP."""
    try:
        with io.BytesIO() as fl:
            with target.open_sftp() as sftp:
                sftp.getfo(remote_path, fl)
                fl.seek(0)
                return fl.read()
    except Exception:
        return None



# !/usr/bin/python
# coding=utf-8
import shutil
import subprocess
from pathlib import Path

import lief

from mobsf.StaticAnalyzer.views.common.binary.strings import (
    strings_on_binary,
)
from mobsf.MobSF.utils import (
    run_with_timeout,
)

from django.conf import settings


def objdump_is_debug_symbol_stripped(macho_file):
    """Check if debug symbols are stripped using OS utility."""
    # https://www.unix.com/man-page/osx/1/objdump/
    # Works only on MacOS
    out = subprocess.check_output(
        [shutil.which('objdump'), '--syms', macho_file],
        stderr=subprocess.STDOUT)
    out = out.decode('utf-8', 'ignore')
    # radr://5614542 symbol is added back for
    # debug symbols stripped binaries
    stripped_sym = 'radr://5614542'
    for line in out.splitlines():
        if ' d  ' in line and stripped_sym not in line:
            return False
    return True


class MachOChecksec:
    def __init__(self, macho, rel_path=None):
        self.macho_path = macho.as_posix()
        if rel_path:
            self.macho_name = rel_path
        else:
            self.macho_name = macho.name
        self.macho = run_with_timeout(
            lief.parse,
            settings.BINARY_ANALYSIS_TIMEOUT,
            self.macho_path)

    def checksec(self):
        macho_dict = {}
        macho_dict['name'] = self.macho_name

        if not self.is_macho(self.macho_path):
            return {}

        has_nx = self.has_nx()
        has_pie = self.has_pie()
        has_canary = self.has_canary()
        has_rpath = self.has_rpath()
        has_code_signature = self.has_code_signature()
        has_arc = self.has_arc()
        is_encrypted = self.is_encrypted()
        is_stripped = self.is_symbols_stripped()

        if has_nx:
            severity = 'info'
            desc = (
                'The binary has NX bit set. This marks a '
                'memory page non-executable making attacker '
                'injected shellcode non-executable.')
        else:
            severity = 'info'
            desc = (
                'The binary does not have NX bit set. NX bit '
                'offer protection against exploitation of memory corruption '
                'vulnerabilities by marking memory page as non-executable. '
                'However iOS never allows an app to execute from writeable '
                'memory. You do not need to specifically enable the '
                '‘NX bit’ because it’s always enabled for all '
                'third-party code.')
        macho_dict['nx'] = {
            'has_nx': has_nx,
            'severity': severity,
            'description': desc,
        }
        if has_pie:
            severity = 'info'
            desc = (
                'The binary is build with -fPIC flag which '
                'enables Position independent code. This makes Return '
                'Oriented Programming (ROP) attacks much more difficult '
                'to execute reliably.')
        else:
            severity = 'high'
            ext = Path(self.macho_name).suffix
            # PIE check not applicable for static and dynamic libraries
            # https://github.com/MobSF/Mobile-Security-Framework-MobSF/
            # issues/2290#issuecomment-1837272113
            if (ext == '.dylib'
                    or (not ext and '.framework' in self.macho_name)):
                severity = 'info'
            desc = (
                'The binary is built without Position '
                'Independent Code flag. In order to prevent '
                'an attacker from reliably jumping to, for example, a '
                'particular exploited function in memory, Address '
                'space layout randomization (ASLR) randomly arranges '
                'the address space positions of key data areas of a '
                'process, including the base of the executable and the '
                'positions of the stack,heap and libraries. Use compiler '
                'option -fPIC to enable Position Independent Code. '
                'Not applicable for dylibs and static libraries.')
        macho_dict['pie'] = {
            'has_pie': has_pie,
            'severity': severity,
            'description': desc,
        }
        if has_canary:
            severity = 'info'
            desc = (
                'This binary has a stack canary value '
                'added to the stack so that it will be overwritten by '
                'a stack buffer that overflows the return address. '
                'This allows detection of overflows by verifying the '
                'integrity of the canary before function return.')
        elif is_stripped:
            severity = 'warning'
            desc = (
                'This binary has debug symbols stripped. We cannot identify '
                'whether stack canary is enabled or not.')
        else:
            severity = 'high'
            sw_msg = ''
            if 'libswift' in self.macho_name:
                severity = 'warning'
                sw_msg = ' This might be okey for pure Swift dylibs.'
            desc = (
                'This binary does not have a stack '
                'canary value added to the stack. Stack canaries '
                'are used to detect and prevent exploits from '
                'overwriting return address. Use the option '
                f'-fstack-protector-all to enable stack canaries.{sw_msg}')
        macho_dict['stack_canary'] = {
            'has_canary': has_canary,
            'severity': severity,
            'description': desc,
        }
        if has_arc:
            severity = 'info'
            desc = (
                'The binary is compiled with Automatic Reference '
                'Counting (ARC) flag. ARC is a compiler '
                'feature that provides automatic memory '
                'management of Objective-C objects and is an '
                'exploit mitigation mechanism against memory '
                'corruption vulnerabilities.'
            )
        elif is_stripped:
            severity = 'warning'
            desc = (
                'This binary has debug symbols stripped. We cannot identify '
                'whether ARC is enabled or not.')
        else:
            severity = 'high'
            desc = (
                'The binary is not compiled with Automatic '
                'Reference Counting (ARC) flag. ARC is a compiler '
                'feature that provides automatic memory '
                'management of Objective-C objects and '
                'protects from memory corruption '
                'vulnerabilities. Use compiler option '
                '-fobjc-arc to enable ARC or set '
                'Objective-C Automatic Reference Counting '
                'to YES in project configuration.')
        macho_dict['arc'] = {
            'has_arc': has_arc,
            'severity': severity,
            'description': desc,
        }
        if has_rpath:
            severity = 'warning'
            desc = (
                'The binary has Runpath Search Path (@rpath) set. '
                'In certain cases an attacker can abuse this '
                'feature to run arbitrary executable for code '
                'execution and privilege escalation. Remove the '
                'compiler option -rpath to remove @rpath.')
        else:
            severity = 'info'
            desc = (
                'The binary does not have Runpath Search '
                'Path (@rpath) set.')
        macho_dict['rpath'] = {
            'has_rpath': has_rpath,
            'severity': severity,
            'description': desc,
        }
        if has_code_signature:
            severity = 'info'
            desc = 'This binary has a code signature.'
        else:
            severity = 'warning'
            desc = 'This binary does not have a code signature.'
        macho_dict['code_signature'] = {
            'has_code_signature': has_code_signature,
            'severity': severity,
            'description': desc,
        }
        if is_encrypted:
            severity = 'info'
            desc = 'This binary is encrypted.'
        else:
            severity = 'warning'
            desc = 'This binary is not encrypted.'
        macho_dict['encrypted'] = {
            'is_encrypted': is_encrypted,
            'severity': severity,
            'description': desc,
        }
        if is_stripped:
            severity = 'info'
            desc = 'Debug Symbols are stripped'
        else:
            severity = 'warning'
            desc = (
                'Debug Symbols are available. To strip '
                'debugging symbols, set Strip Debug '
                'Symbols During Copy to YES, '
                'Deployment Postprocessing to YES, '
                'and Strip Linked Product to YES in '
                'project\'s build settings.')
        macho_dict['symbol'] = {
            'is_stripped': is_stripped,
            'severity': severity,
            'description': desc,
        }
        return macho_dict

    def is_macho(self, macho_path):
        return lief.is_macho(macho_path)

    def has_nx(self):
        return self.macho.has_nx

    def has_pie(self):
        return self.macho.is_pie

    def has_canary(self):
        stk_check = '___stack_chk_fail'
        stk_guard = '___stack_chk_guard'
        imp_func_gen = self.macho.imported_functions
        has_stk_check = any(
            str(func.name).strip() == stk_check for func in imp_func_gen)
        has_stk_guard = any(
            str(func.name).strip() == stk_guard for func in imp_func_gen)

        return has_stk_check and has_stk_guard

    def has_arc(self):
        arc_funcs = ('_objc_release', '_swift_release')
        for func in self.macho.imported_functions:
            if str(func.name).strip() in arc_funcs:
                return True
        return False

    def has_rpath(self):
        return self.macho.has_rpath

    def has_code_signature(self):
        try:
            return self.macho.code_signature.data_size > 0
        except Exception:
            return False

    def is_encrypted(self):
        try:
            return bool(self.macho.encryption_info.crypt_id)
        except Exception:
            return False

    def is_symbols_stripped(self):
        try:
            return objdump_is_debug_symbol_stripped(self.macho_path)
        except Exception:
            # Based on issues/1917#issuecomment-1238078359
            # and issues/2233#issue-1846914047
            stripped_sym = 'radr://5614542'
            # radr://5614542 symbol is added back for
            # debug symbols stripped binaries
            for i in self.macho.symbols:
                if i.name.lower().strip() in (
                        '__mh_execute_header', stripped_sym):
                    # __mh_execute_header is present in both
                    # stripped and unstripped binaries
                    # also ignore radr://5614542
                    continue
                if (i.type.value & 0xe0) > 0 or i.type.value in (0x0e, 0x1e):
                    # N_STAB set or 14, 30

                    # N_STAB	0xe0  /* if any of these bits set,
                    # a symbolic debugging entry */ -> 224
                    # https://opensource.apple.com/source/xnu/xnu-201/
                    # EXTERNAL_HEADERS/mach-o/nlist.h
                    # Only symbolic debugging entries have
                    # some of the N_STAB bits set and if any
                    # of these bits are set then it is a
                    # symbolic debugging entry (a stab).

                    # Identified a debugging symbol
                    return False
            if stripped_sym in self.get_symbols():
                return True
            return False

    def get_libraries(self):
        libs = []
        if not self.macho:
            return libs
        for i in self.macho.libraries:
            curr = '.'.join(str(x) for x in i.current_version)
            comp = '.'.join(str(x) for x in i.compatibility_version)
            lib = (f'{i.name} (compatibility version: {comp}'
                   f', current version: {curr})')
            libs.append(lib)
        return libs

    def strings(self):
        return strings_on_binary(self.macho_path)

    def get_symbols(self):
        symbols = []
        try:
            for i in self.macho.symbols:
                symbols.append(i.name)
        except Exception:
            pass
        return symbols



# -*- coding: utf_8 -*-
"""iOS Analysis."""
import logging
from pathlib import Path

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal

from django.conf import settings
from django.shortcuts import render

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.models import (
    StaticAnalyzerIOS,
)
from mobsf.StaticAnalyzer.views.ios.appstore import app_search
from mobsf.StaticAnalyzer.views.ios.binary_analysis import (
    binary_analysis,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.ios.code_analysis import ios_source_analysis
from mobsf.StaticAnalyzer.views.ios.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.StaticAnalyzer.views.ios.file_analysis import ios_list_files
from mobsf.StaticAnalyzer.views.ios.icon_analysis import (
    get_icon_from_ipa,
    get_icon_source,
)
from mobsf.StaticAnalyzer.views.ios.plist_analysis import (
    get_plist_secrets,
    plist_analysis,
)
from mobsf.StaticAnalyzer.views.ios.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_avg_cvss,
    hash_gen,
    strings_and_entropies,
    unzip,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.StaticAnalyzer.views.common.appsec import (
    get_ios_dashboard,
)
from mobsf.StaticAnalyzer.views.common.async_task import (
    async_analysis,
    mark_task_completed,
    mark_task_started,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import (
    MalwareDomainCheck,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)


def initialize_app_dic(app_dic, file_ext):
    """Initialize App Dictionary."""
    checksum = app_dic['md5_hash']
    app_dic['app_file'] = f'{checksum}.{file_ext}'
    app_dic['app_path'] = (app_dic['app_dirp'] / app_dic['app_file']).as_posix()
    return checksum


def get_size_and_hashes(app_dic):
    app_dic['size'] = str(file_size(app_dic['app_path'])) + 'MB'
    app_dic['sha1'], app_dic['sha256'] = hash_gen(
        app_dic['md5_hash'], app_dic['app_path'])


def extract_and_check_ipa(checksum, app_dic):
    """Extract and Check IPA."""
    # EXTRACT IPA
    msg = 'Extracting IPA'
    logger.info(msg)
    append_scan_status(checksum, msg)
    unzip(
        checksum,
        app_dic['app_path'],
        app_dic['app_dir'])
    # Identify Payload directory
    dirs = app_dic['app_dirp'].glob('**/*')
    for _dir in dirs:
        if 'payload' in _dir.as_posix().lower():
            app_dic['bin_dir'] = app_dic['app_dirp'] / _dir
            break
    else:
        return False
    app_dic['bin_dir'] = app_dic['bin_dir'].as_posix() + '/'
    return True


def common_analysis(scan_type, app_dic, checksum):
    """Common Analysis for ipa and zip."""
    location = app_dic['app_dir']
    if scan_type == 'ipa':
        location = app_dic['bin_dir']
    # Get Files
    app_dic['all_files'] = ios_list_files(
        checksum,
        location,
        scan_type)
    # Plist files are converted to xml/readable for ipa
    app_dic['infoplist'] = plist_analysis(
        checksum,
        location,
        scan_type)
    app_dic['appstore'] = app_search(
        checksum,
        app_dic['infoplist'].get('id'))
    app_dic['secrets'] = get_plist_secrets(
        checksum,
        location)


def common_firebase_and_trackers(code_dict, app_dic, checksum):
    """Common Firebase and Trackers."""
    # Firebase Analysis
    code_dict['firebase'] = firebase_analysis(
        checksum,
        code_dict)
    # Extract Trackers from Domains
    trk = Trackers.Trackers(
        checksum,
        None,
        app_dic['tools_dir'])
    code_dict['trackers'] = trk.get_trackers_domains_or_deps(
        code_dict['domains'], [])


def get_scan_subject(app_dic, bin_dict):
    """Get Scan Subject."""
    app_name = None
    pkg_name = None
    subject = 'iOS App'
    if bin_dict.get('bin_path'):
        app_name = bin_dict['bin_path'].name if bin_dict['bin_path'] else None
    if app_dic.get('infoplist'):
        pkg_name = app_dic['infoplist'].get('id')

    if app_name and pkg_name:
        subject = f'{app_name} ({pkg_name})'
    elif pkg_name:
        subject = pkg_name
    elif app_name:
        subject = app_name
    if subject == 'Failed':
        subject = f'({subject})'
    return subject


def ipa_analysis_task(checksum, app_dic, rescan, queue=False):
    """IPA Analysis Task."""
    context = None
    try:
        if queue:
            settings.ASYNC_ANALYSIS = True
            mark_task_started(checksum)
        append_scan_status(checksum, 'init')
        msg = 'iOS Binary (IPA) Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        get_size_and_hashes(app_dic)

        if not extract_and_check_ipa(checksum, app_dic):
            msg = ('IPA is malformed! MobSF cannot find Payload directory')
            append_scan_status(checksum, 'IPA is malformed', msg)
            if queue:
                return mark_task_completed(
                    checksum, 'Failed', msg)
            return context, msg

        # Common Analysis
        common_analysis('ipa', app_dic, checksum)
        # IPA Binary Analysis
        bin_dict = binary_analysis(
            checksum,
            app_dic['bin_dir'],
            app_dic['tools_dir'],
            app_dic['app_dir'],
            app_dic['infoplist'].get('bin'))
        # Analyze dylibs and frameworks
        lb = library_analysis(
            checksum,
            app_dic['bin_dir'],
            'macho')
        bin_dict['dylib_analysis'] = lb['macho_analysis']
        bin_dict['framework_analysis'] = lb['framework_analysis']
        # Extract String metadata from binary
        code_dict = get_strings_metadata(
            app_dic,
            bin_dict,
            app_dic['all_files'],
            lb['macho_strings'])
        # Domain Extraction and Malware Check
        code_dict['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_dict['urls_list'])
        # Get Icon
        get_icon_from_ipa(app_dic)
        # Firebase and Trackers
        common_firebase_and_trackers(code_dict, app_dic, checksum)

        code_dict['api'] = {}
        code_dict['code_anal'] = {}
        context = save_get_ctx(
            app_dic,
            code_dict,
            bin_dict,
            rescan)
        if queue:
            subject = get_scan_subject(app_dic, bin_dict)
            return mark_task_completed(
                checksum, subject, 'Success')
        return context, None
    except Exception as exp:
        if queue:
            return mark_task_completed(
                checksum, 'Failed', repr(exp))
        return context, repr(exp)


def generate_dynamic_context(request, app_dic, context, checksum, api):
    """Generate Dynamic Context."""
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(app_dic['app_path'])
    context['appsec'] = get_ios_dashboard(context, True)
    context['average_cvss'] = get_avg_cvss(context['binary_analysis'])
    template = 'static_analysis/ios_binary_analysis.html'
    return context if api else render(request, template, context)


def ipa_analysis(request, app_dic, rescan, api):
    """IPA Analysis."""
    checksum = initialize_app_dic(app_dic, 'ipa')
    ipa_db = StaticAnalyzerIOS.objects.filter(MD5=checksum)
    if ipa_db.exists() and not rescan:
        context = get_context_from_db_entry(ipa_db)
        return generate_dynamic_context(request, app_dic, context, checksum, api)
    else:
        # IPA Analysis
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(request, 'Permission Denied', False)
        if settings.ASYNC_ANALYSIS:
            return async_analysis(
                checksum,
                api,
                app_dic.get('file_name', ''),
                ipa_analysis_task, checksum, app_dic, rescan)
        context, err = ipa_analysis_task(checksum, app_dic, rescan)
        if err:
            return print_n_send_error_response(request, err, api)
        return generate_dynamic_context(request, app_dic, context, checksum, api)


def ios_analysis_task(checksum, app_dic, rescan, queue=False):
    """IOS Analysis Task."""
    context = None
    try:
        if queue:
            settings.ASYNC_ANALYSIS = True
            mark_task_started(checksum)
        logger.info('iOS Source Code Analysis Started')
        get_size_and_hashes(app_dic)

        # ANALYSIS BEGINS - Already Unzipped
        # append_scan_status init done in android static analyzer
        common_analysis('zip', app_dic, checksum)

        # IOS Source Code Analysis
        code_dict = ios_source_analysis(
            checksum,
            app_dic['app_dir'])
        # Extract Strings and entropies from source code
        ios_strs = strings_and_entropies(
            checksum,
            Path(app_dic['app_dir']),
            ['.swift', '.m', '.h', '.plist', '.json'])
        if ios_strs['secrets']:
            app_dic['secrets'].extend(list(ios_strs['secrets']))
        # Get App Icon
        get_icon_source(app_dic)
        # Firebase and Trackers
        common_firebase_and_trackers(code_dict, app_dic, checksum)

        bin_dict = {
            'checksec': {},
            'libraries': [],
            'bin_code_analysis': {},
            'strings': list(ios_strs['strings']),
            'bin_info': {},
            'bin_type': code_dict['source_type'],
            'dylib_analysis': {},
            'framework_analysis': {},
        }
        context = save_get_ctx(
            app_dic,
            code_dict,
            bin_dict,
            rescan)
        if queue:
            subject = get_scan_subject(app_dic, bin_dict)
            return mark_task_completed(
                checksum, subject, 'Success')
    except Exception as exp:
        if queue:
            return mark_task_completed(
                checksum, 'Failed', repr(exp))
    return context


def generate_dynamic_ios_context(request, context, api):
    """Generate Dynamic Context for IOS."""
    context['appsec'] = get_ios_dashboard(context, True)
    context['average_cvss'] = get_avg_cvss(context['code_analysis'])
    template = 'static_analysis/ios_source_analysis.html'
    return context if api else render(request, template, context)


def ios_analysis(request, app_dic, rescan, api):
    """IOS Source Code Analysis."""
    checksum = initialize_app_dic(app_dic, 'zip')
    ios_zip_db = StaticAnalyzerIOS.objects.filter(MD5=checksum)
    if ios_zip_db.exists() and not rescan:
        context = get_context_from_db_entry(ios_zip_db)
        return generate_dynamic_ios_context(request, context, api)
    else:
        # IOS Source Analysis
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(request, 'Permission Denied', False)
        if settings.ASYNC_ANALYSIS:
            return async_analysis(
                checksum,
                api,
                app_dic.get('file_name', ''),
                ios_analysis_task, checksum, app_dic, rescan)
        context = ios_analysis_task(checksum, app_dic, rescan)
        return generate_dynamic_ios_context(request, context, api)



import re
from pathlib import Path

from django import forms

from mobsf.MobSF.security import is_path_traversal
from mobsf.MobSF.utils import (
    is_md5,
)


class AttackDetect(forms.Form):
    file = forms.CharField()

    def clean_file(self):
        """Check for path traversal."""
        file = self.cleaned_data['file']
        if is_path_traversal(file):
            raise forms.ValidationError('Attack Detected')
        # Allowed File extensions
        supported_ext = (r'^\.(kt|java|smali|xml|'
                         r'plist|m|swift|'
                         r'db|sqlitedb|sqlite|txt|json)$')
        if not re.search(supported_ext, Path(file).suffix):
            raise forms.ValidationError('File Extension not supported')
        return file


class IOSChecks(forms.Form):
    type = forms.ChoiceField(  # noqa A003
        choices=(
            ('ipa', 'ipa'),
            ('dylib', 'dylib'),
            ('ios', 'ios')))


class APIChecks(forms.Form):
    hash = forms.CharField(min_length=32, max_length=32)  # noqa A003

    def clean_hash(self):
        """Hash is valid."""
        md5 = self.cleaned_data['hash']
        if not is_md5(md5):
            raise forms.ValidationError('Invalid Hash')
        return md5


class WebChecks(forms.Form):
    md5 = forms.CharField(min_length=32, max_length=32)

    def clean_md5(self):
        """Hash is valid."""
        md5 = self.cleaned_data['md5']
        if not is_md5(md5):
            raise forms.ValidationError('Invalid Hash')
        return md5


class AndroidChecks(forms.Form):
    type = forms.ChoiceField(  # noqa A003
        choices=(
            ('eclipse', 'eclipse'),
            ('studio', 'studio'),
            ('java', 'java'),
            ('smali', 'smali'),
            ('apk', 'apk'),
            ('jar', 'jar'),
            ('aar', 'aar'),
            ('so', 'so'),
            ('a', 'a')))


class ViewSourceIOSApiForm(AttackDetect, IOSChecks, APIChecks):
    pass


class ViewSourceIOSForm(AttackDetect, IOSChecks, WebChecks):
    pass


class ViewSourceAndroidApiForm(AttackDetect, AndroidChecks, APIChecks):
    pass


class ViewSourceAndroidForm(AttackDetect, AndroidChecks, WebChecks):
    pass



# -*- coding: utf_8 -*-
"""iOS Static Code Analysis."""
import logging
from pathlib import Path

from django.conf import settings
from django.template.defaulttags import register

from mobsf.MobSF.utils import (
    append_scan_status,
    is_md5,
    print_n_send_error_response,
    relative_path,
)
from mobsf.StaticAnalyzer.models import (
    RecentScansDB,
)
from mobsf.StaticAnalyzer.views.common.a import (
    a_analysis,
)
from mobsf.StaticAnalyzer.views.ios.dylib import (
    dylib_analysis,
)
from mobsf.StaticAnalyzer.views.ios.ipa import (
    ios_analysis,
    ipa_analysis,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)

logger = logging.getLogger(__name__)
register.filter('relative_path', relative_path)


@login_required
def static_analyzer_ios(request, checksum, api=False):
    """Module that performs iOS IPA/ZIP Static Analysis."""
    try:
        rescan = False
        if api:
            re_scan = request.POST.get('re_scan', 0)
        else:
            re_scan = request.GET.get('rescan', 0)
        if re_scan == '1':
            rescan = True
        app_dict = {}
        if not is_md5(checksum):
            return print_n_send_error_response(
                request,
                'Invalid Hash',
                api)
        robj = RecentScansDB.objects.filter(MD5=checksum)
        if not robj.exists():
            return print_n_send_error_response(
                request,
                'The file is not uploaded/available',
                api)
        file_type = robj[0].SCAN_TYPE
        filename = robj[0].FILE_NAME
        if file_type == 'dylib' and not Path(filename).suffix:
            # Force dylib extension on Frameworks
            filename = f'{filename}.dylib'
        ios_exts = tuple(f'.{i}' for i in settings.IOS_EXTS)
        allowed_exts = ios_exts + ('.zip', 'ios')
        allowed_types = settings.IOS_EXTS + ('zip', 'ios')
        if (not filename.lower().endswith(allowed_exts)
                or file_type not in allowed_types):
            return print_n_send_error_response(
                request,
                'Invalid file extension or file type',
                api)
        app_dict['directory'] = Path(settings.BASE_DIR)  # BASE DIR
        app_dict['file_name'] = filename  # APP ORIGINAL NAME
        app_dict['md5_hash'] = checksum  # MD5
        app_dict['app_dirp'] = Path(settings.UPLD_DIR) / checksum
        app_dict['app_dir'] = app_dict['app_dirp'].as_posix() + '/'
        tools_dir = app_dict[
            'directory'] / 'StaticAnalyzer' / 'tools' / 'ios'
        app_dict['tools_dir'] = tools_dir.as_posix()
        app_dict['icon_path'] = ''
        if file_type == 'ipa':
            return ipa_analysis(request, app_dict, rescan, api)
        elif file_type == 'dylib':
            return dylib_analysis(request, app_dict, rescan, api)
        elif file_type == 'a':
            return a_analysis(request, app_dict, rescan, api)
        elif file_type in ('ios', 'zip'):
            return ios_analysis(request, app_dict, rescan, api)
        else:
            err = ('File Type not supported, '
                   'Only IPA, A, DYLIB and ZIP are supported')
            logger.error(err)
            append_scan_status(checksum, err)
            raise Exception(err)
    except Exception as exp:
        msg = 'Error Performing Static Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
        exp_doc = exp.__doc__
        return print_n_send_error_response(request, repr(exp), api, exp_doc)



# -*- coding: utf_8 -*-
"""View Source of a file."""
import logging
import ntpath
from pathlib import Path

from django.conf import settings
from django.shortcuts import render
from django.utils.html import escape
from django.http import JsonResponse

from mobsf.MobSF.forms import FormUtil
from mobsf.MobSF.security import is_safe_path
from mobsf.MobSF.utils import (
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    find_java_source_folder,
)
from mobsf.StaticAnalyzer.forms import (
    ViewSourceAndroidApiForm,
    ViewSourceAndroidForm,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)

logger = logging.getLogger(__name__)


def send_json(data):
    return JsonResponse(data, safe=False)


def send_error(request, err, api_mode, json_resp, exp=None):
    """Send error message as dict or JSON."""
    if exp:
        res = print_n_send_error_response(request, err, api_mode, exp)
    else:
        res = print_n_send_error_response(request, err, api_mode)
    if json_resp:
        return send_json(res)
    return res


@login_required
def run(request, api=False):
    """View the source of a file."""
    json_resp = request.GET.get('json', '0') == '1'
    api_mode = api or json_resp
    try:
        logger.info('View Java Source File')
        exp = 'Error Description'
        if api:
            fil = request.POST['file']
            md5 = request.POST['hash']
            typ = request.POST['type']
            viewsource_form = ViewSourceAndroidApiForm(request.POST)
        else:
            fil = request.GET['file']
            md5 = request.GET['md5']
            typ = request.GET['type']
            viewsource_form = ViewSourceAndroidForm(request.GET)
        if not viewsource_form.is_valid():
            err = FormUtil.errors_message(viewsource_form)
            return send_error(request, err, api_mode, json_resp, exp)
        base = Path(settings.UPLD_DIR) / md5
        if typ == 'smali':
            src = base / 'smali_source'
            syntax = 'smali'
        else:
            try:
                src, syntax, _ = find_java_source_folder(base)
            except StopIteration:
                msg = 'Invalid directory or file extension'
                return send_error(request, msg, api_mode, json_resp)

        sfile = src / fil
        if not is_safe_path(src, sfile.as_posix(), fil):
            msg = 'Path Traversal Detected!'
            return send_error(request, msg, api_mode, json_resp)
        context = {
            'title': escape(ntpath.basename(fil)),
            'file': escape(ntpath.basename(fil)),
            'data': sfile.read_text('utf-8', 'ignore'),
            'type': syntax,
            'sqlite': {},
            'version': settings.MOBSF_VER,
        }
        template = 'general/view.html'
        if json_resp:
            return send_json(context)
        if api_mode:
            return context
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Error Viewing Source')
        msg = str(exp)
        exp = exp.__doc__
        return send_error(request, msg, api_mode, json_resp, exp)



# -*- coding: utf_8 -*-
"""Windows Analysis Module."""
import base64
# Local analysis
import configparser
import json
import logging
import os
import platform
import re
import subprocess
# Binskim/Binscope analysis
import xmlrpc.client
from os.path import expanduser

import rsa

from lxml import etree


from django.conf import settings
from django.shortcuts import render
from django.utils.html import escape

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    get_config_loc,
    is_md5,
    print_n_send_error_response,
)
from mobsf.MobSF.views.home import update_scan_timestamp
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal
from mobsf.StaticAnalyzer.models import (
    RecentScansDB,
    StaticAnalyzerWindows,
)
from mobsf.StaticAnalyzer.tools.strings import strings_util
from mobsf.StaticAnalyzer.views.common.shared_func import (
    hash_gen,
    unzip,
)
from mobsf.StaticAnalyzer.views.windows.db_interaction import (
    get_context_from_analysis,
    get_context_from_db_entry,
    save_or_update,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)
# Only used when xmlrpc is used
proxy = None
# Used to store the local config if windows analysis happens local
config = None

##############################################################
# Code to support Windows Static Code Analysis
##############################################################
# Windows Support Functions


@login_required
def staticanalyzer_windows(request, checksum, api=False):
    """Analyse a windows app."""
    try:
        # Input validation
        logger.info('Windows Static Analysis Started')
        rescan = False
        app_dic = {}  # Dict to store the binary attributes
        if api:
            re_scan = request.POST.get('re_scan', 0)
        else:
            re_scan = request.GET.get('rescan', 0)
        if re_scan == '1':
            rescan = True
        if not is_md5(checksum):
            return print_n_send_error_response(
                request,
                'Invalid Hash',
                api)
        robj = RecentScansDB.objects.filter(MD5=checksum)
        if not robj.exists():
            return print_n_send_error_response(
                request,
                'The file is not uploaded/available',
                api)
        typ = robj[0].SCAN_TYPE
        filename = robj[0].FILE_NAME
        if typ not in settings.WINDOWS_EXTS:
            return print_n_send_error_response(
                request,
                'File type not supported',
                api)

        app_dic['app_name'] = filename  # APP ORIGINAL NAME
        app_dic['md5'] = checksum
        app_dic['app_dir'] = os.path.join(
            settings.UPLD_DIR, checksum + '/')
        app_dic['tools_dir'] = os.path.join(
            settings.BASE_DIR, 'StaticAnalyzer/tools/windows/')
        # DB
        db_entry = StaticAnalyzerWindows.objects.filter(
            MD5=checksum,
        )
        if db_entry.exists() and not rescan:
            logger.info(
                'Analysis is already Done.'
                ' Fetching data from the DB...')
            context = get_context_from_db_entry(db_entry)
        else:
            if not has_permission(request, Permissions.SCAN, api):
                return print_n_send_error_response(
                    request,
                    'Permission Denied',
                    False)
            append_scan_status(checksum, 'init')
            msg = 'Windows Binary Analysis Started'
            logger.info(msg)
            append_scan_status(checksum, msg)
            app_dic['app_path'] = os.path.join(
                app_dic['app_dir'], checksum + '.appx')
            # ANALYSIS BEGINS
            app_dic['size'] = str(
                file_size(app_dic['app_path'])) + 'MB'
            # Generate hashes
            app_dic['sha1'], app_dic['sha256'] = hash_gen(
                checksum,
                app_dic['app_path'])
            # EXTRACT APPX
            logger.info('Extracting APPX')
            app_dic['files'] = unzip(
                checksum,
                app_dic['app_path'],
                app_dic['app_dir'])
            xml_dic = _parse_xml(
                checksum,
                app_dic['app_dir'])
            bin_an_dic = _binary_analysis(app_dic)
            # Saving to db
            logger.info('Connecting to DB')
            if rescan:
                logger.info('Updating Database...')
                save_or_update('update',
                               app_dic,
                               xml_dic,
                               bin_an_dic)
                update_scan_timestamp(checksum)
            else:
                logger.info('Saving to Database')
                save_or_update('save',
                               app_dic,
                               xml_dic,
                               bin_an_dic)
            context = get_context_from_analysis(app_dic,
                                                xml_dic,
                                                bin_an_dic)
            context['virus_total'] = None
        template = 'static_analysis/windows_binary_analysis.html'
        context['virus_total'] = None
        if settings.VT_ENABLED:
            vt = VirusTotal.VirusTotal(checksum)
            context['virus_total'] = vt.get_result(
                os.path.join(app_dic['app_dir'], checksum) + '.appx')
        if api:
            return context
        else:
            return render(request, template, context)
    except Exception as exception:
        msg = 'Error Performing Static Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exception))
        return print_n_send_error_response(
            request,
            repr(exception),
            api,
            exception.__doc__)


def _get_token():
    """Get the authentication token for windows vm xmlrpc client."""
    challenge = proxy.get_challenge()
    priv_key = rsa.PrivateKey.load_pkcs1(
        open(settings.WINDOWS_VM_SECRET).read())
    signature = rsa.sign(challenge.encode('ascii'), priv_key, 'SHA-512')
    sig_b64 = base64.b64encode(signature)
    return sig_b64


def _binary_analysis(app_dic):
    """Start binary analsis."""
    msg = 'Starting Binary Analysis'
    logger.info(msg)
    append_scan_status(app_dic['md5'], msg)
    bin_an_dic = {}

    # Init optional sections to prevent None-Pointer-Errors
    bin_an_dic['results'] = []
    bin_an_dic['warnings'] = []
    # Search for exe
    for file_name in app_dic['files']:
        if file_name.endswith('.exe'):
            bin_an_dic['bin'] = file_name
            bin_an_dic['bin_name'] = file_name.replace('.exe', '')
            break
    if not bin_an_dic.get('bin_name'):
        logger.error('No executable in appx')
    bin_path = os.path.join(app_dic['app_dir'], bin_an_dic['bin'])

    # Execute strings command
    bin_an_dic['strings'] = ''
    # Make unique # pylint: disable-msg=R0204
    str_list = list(set(strings_util(bin_path)))
    str_list = [escape(s) for s in str_list]
    bin_an_dic['strings'] = str_list

    # Search for unsave function
    pattern = re.compile('(alloca|gets|memcpy|printf|scanf|sprintf|sscanf|'
                         'strcat|StrCat|strcpy|StrCpy|strlen|StrLen|strncat|'
                         'StrNCat|strncpy|StrNCpy|strtok|swprintf|vsnprintf|'
                         'vsprintf|vswprintf|wcscat|wcscpy|wcslen|wcsncat|'
                         'wcsncpy|wcstok|wmemcpy)')
    for elem in str_list:
        if pattern.match(elem[5:-5]):
            result = {
                'rule_id': 'Possible Insecure Function',
                'status': 'Insecure',
                'desc': ('Possible Insecure '
                         'Function detected: {}').format(elem[5:-5])}
            bin_an_dic['results'].append(result)

    # Execute binskim analysis if vm is available
    if platform.system() != 'Windows' or 'CI' in os.environ:
        if settings.WINDOWS_VM_IP:
            msg = 'Windows VM configured'
            logger.info(msg)
            append_scan_status(app_dic['md5'], msg)
            global proxy
            proxy = xmlrpc.client.ServerProxy(  # pylint: disable-msg=C0103
                'http://{}:{}'.format(
                    settings.WINDOWS_VM_IP,
                    settings.WINDOWS_VM_PORT))
            name = _upload_sample(bin_path)
            bin_an_dic = binskim(app_dic['md5'], name, bin_an_dic)
            bin_an_dic = binscope(app_dic['md5'], name, bin_an_dic)
        else:
            msg = (
                f'Windows VM not configured in {get_config_loc()}.'
                ' Skipping Binskim and Binscope analysis')
            logger.warning(msg)
            append_scan_status(app_dic['md5'], msg)
            warning = {
                'rule_id': 'VM',
                'status': 'Info',
                'info': '',
                'desc': 'VM is not configured. Please read the readme.md'
                ' in MobSF/install/windows.',
            }
            bin_an_dic['results'].append(warning)
    else:
        msg = 'Running Analysis on Windows host'
        logger.info(msg)
        append_scan_status(app_dic['md5'], msg)
        global config
        config = configparser.ConfigParser()
        # Switch to settings defined path if available
        config.read(expanduser('~') + '\\MobSF\\Config\\config.txt')

        # Run analysis functions
        bin_an_dic = binskim(app_dic['md5'],
                             bin_path,
                             bin_an_dic,
                             run_local=True,
                             app_dir=app_dic['app_dir'])
        bin_an_dic = binscope(app_dic['md5'],
                              bin_path,
                              bin_an_dic,
                              run_local=True,
                              app_dir=app_dic['app_dir'])

    return bin_an_dic


def _upload_sample(bin_path):
    """Upload sample to windows vm."""
    logger.info('Uploading sample.')

    # Upload test
    with open(bin_path, 'rb') as handle:
        binary_data = xmlrpc.client.Binary(handle.read())
    # Name of the sample is return by the remote machine
    name = proxy.upload_file(binary_data, _get_token())

    return name


def binskim(checksum, name, bin_an_dic, run_local=False, app_dir=None):
    """Run the binskim analysis."""
    msg = 'Running binskim.'
    logger.info(msg)
    append_scan_status(checksum, msg)
    if run_local:
        bin_path = os.path.join(app_dir, bin_an_dic['bin'])

        # Set params for execution of binskim
        if platform.machine().endswith('64'):
            binskim_path = config['binskim']['file_x64']
        else:
            binskim_path = config['binskim']['file_x86']

        command = 'analyze'
        path = bin_path
        output_p = '-o'
        output_d = bin_path + '_binskim'
        verbose = '--verbose'
        policy_p = '--config'
        force = '--force'
        policy_d = 'default'  # TODO(Other policies?)

        # Assemble
        params = [
            binskim_path,
            command,
            path,
            verbose,
            output_p, output_d,
            policy_p, policy_d, force,
        ]

        # Execute process
        pipe = subprocess.Popen(subprocess.list2cmdline(params))
        pipe.wait()  # Wait for the process to finish..

        # Open the file and return the json
        out_file = open(output_d)
        output = json.loads(out_file.read())
    else:
        # Analyse the sample
        response = proxy.binskim(name, _get_token())

        # Load output as json
        output = json.loads(response)
    bin_an_dic = parse_binskim(bin_an_dic, output)
    return bin_an_dic


def parse_binskim(bin_an_dic, output):
    """Parse output to results and warnings."""
    try:
        output['runs'][0]['rules']
        return parse_binskim_old(bin_an_dic, output)
    except Exception:
        return parse_binskim_sarif(bin_an_dic, output)


def get_short_desc(rules, rule_id):
    """Get short description from sarif."""
    for i in rules:
        if i['id'] == rule_id:
            return i['shortDescription']['text']


def parse_binskim_sarif(bin_an_dic, output):
    """Parse new version of binskim sarif output."""
    current_run = output['runs'][0]
    rules = current_run['tool']['driver']['rules']
    if 'results' in current_run:
        for res in current_run['results']:
            if res['level'] != 'pass':
                if len(res['message']['arguments']) > 2:
                    info = ('{}, {}').format(
                        res['message']['arguments'][1],
                        res['message']['arguments'][2])
                else:
                    info = ''
                result = {
                    'rule_id': res['ruleId'],
                    'status': 'Insecure',
                    'info': info,
                    'desc': get_short_desc(rules, res['ruleId']),
                }
            else:
                result = {
                    'rule_id': res['ruleId'],
                    'status': 'Secure',
                    'info': '',
                    'desc': get_short_desc(rules, res['ruleId']),
                }
            bin_an_dic['results'].append(result)
    else:
        logger.warning('binskim has no results.')
        # Create an warning for the gui
        warning = {
            'rule_id': 'No Binskim-Results',
            'status': 'Info',
            'info': '',
            'desc': 'No results from Binskim.',
        }
        bin_an_dic['warnings'].append(warning)
    return bin_an_dic


def parse_binskim_old(bin_an_dic, output):
    """Parse old version of binskim."""
    current_run = output['runs'][0]
    if 'results' in current_run:
        rules = output['runs'][0]['rules']
        for res in current_run['results']:
            if res['level'] != 'pass':
                if len(res['formattedRuleMessage']['arguments']) > 2:
                    info = ('{}, {}').format(
                        res['formattedRuleMessage']['arguments'][1],
                        res['formattedRuleMessage']['arguments'][2])
                else:
                    info = ''
                result = {
                    'rule_id': res['ruleId'],
                    'status': 'Insecure',
                    'info': info,
                    'desc': rules[res['ruleId']]['shortDescription'],
                }
            else:
                result = {
                    'rule_id': res['ruleId'],
                    'status': 'Secure',
                    'info': '',
                    'desc': rules[res['ruleId']]['shortDescription'],
                }
            bin_an_dic['results'].append(result)
    else:
        logger.warning('binskim has no results.')
        # Create an warning for the gui
        warning = {
            'rule_id': 'No Binskim-Results',
            'status': 'Info',
            'info': '',
            'desc': 'No results from Binskim.',
        }
        bin_an_dic['warnings'].append(warning)

    if 'configurationNotifications' in current_run:
        for warn in current_run['configurationNotifications']:
            warning = {
                'rule_id': warn['ruleId'],
                'status': 'Info',
                'info': '',
                'desc': warn['message'],
            }
            bin_an_dic['warnings'].append(warning)

    # Return updated dict
    return bin_an_dic


def binscope(checksum, name, bin_an_dic, run_local=False, app_dir=None):
    """Run the binskim analysis."""
    msg = 'Running binscope. This might take a while.'
    logger.info(msg)
    append_scan_status(checksum, msg)
    if run_local:
        global config
        bin_path = os.path.join(app_dir, bin_an_dic['bin'])

        # Set params for execution of binskim
        binscope_path = [config['binscope']['file']]
        target = [bin_path]
        out_type = ['/Red', '/v']
        output = ['/l', target[0] + '_binscope']
        checks = [
            '/Checks', 'ATLVersionCheck',
            '/Checks', 'ATLVulnCheck',
            '/Checks', 'AppContainerCheck',
            '/Checks', 'CompilerVersionCheck',
            '/Checks', 'DBCheck',
            '/Checks', 'DefaultGSCookieCheck',
            '/Checks', 'ExecutableImportsCheck',
            '/Checks', 'FunctionPointersCheck',
            '/Checks', 'GSCheck',
            '/Checks', 'GSFriendlyInitCheck',
            '/Checks', 'GSFunctionSafeBuffersCheck',
            '/Checks', 'HighEntropyVACheck',
            '/Checks', 'NXCheck',
            '/Checks', 'RSA32Check',
            '/Checks', 'SafeSEHCheck',
            '/Checks', 'SharedSectionCheck',
            '/Checks', 'VB6Check',
            '/Checks', 'WXCheck',
        ]

        # Assemble
        params = (
            binscope_path
            + target
            + out_type
            + output
            + checks
        )

        # Execute process
        p = subprocess.Popen(subprocess.list2cmdline(params))
        p.wait()  # Wait for the process to finish..

        # Open the file and return the json
        f = open(output[1])
        response = f.read()
    else:
        # Analyse the sample via rpc
        response = proxy.binscope(name, _get_token())

    res = response[response.find('<'):]
    config = etree.XMLParser(  # pylint: disable-msg=E1101
        remove_blank_text=True,
        resolve_entities=False,
    )
    xml_file = etree.XML(bytes(res, 'utf-8', 'ignore'),
                         config)  # pylint: disable-msg=E1101

    for item in xml_file.find('items').getchildren():
        if item.find('issueType') is not None:
            res = item.find('result').text

            if res == 'PASS':
                status = 'Secure'
                try:
                    desc = item.find('Information').text
                except AttributeError:
                    desc = 'No description provided by analysing tool.'
            elif res == 'FAIL':
                status = 'Insecure'

                if item.find('Failure1') is not None:
                    desc = item.find('Failure1').text
                elif item.find('Information') is not None:
                    desc = item.find('Information').text
                elif item.find('diagnostic') is not None:
                    status = 'Info'
                    desc = item.find('diagnostic').text
                else:
                    desc = 'No description provided by analysing tool.'

            result = {
                'rule_id': item.find('issueType').text,
                'status': status,
                'info': '',
                'desc': desc,
            }
            bin_an_dic['results'].append(result)

    return bin_an_dic


def _parse_xml(checksum, app_dir):
    """Parse the AppxManifest file to get basic information."""
    msg = 'Starting Binary Analysis - XML'
    logger.info(msg)
    append_scan_status(checksum, msg)
    xml_file = os.path.join(app_dir, 'AppxManifest.xml')
    xml_dic = {
        'version': '',
        'arch': '',
        'app_name': '',
        'pub_name': '',
        'compiler_version': '',
        'visual_studio_version': '',
        'visual_studio_edition': '',
        'target_os': '',
        'appx_dll_version': '',
        'proj_guid': '',
        'opti_tool': '',
        'target_run': '',
    }

    try:
        msg = 'Reading AppxManifest'
        logger.info(msg)
        append_scan_status(checksum, msg)
        config = etree.XMLParser(  # pylint: disable-msg=E1101
            remove_blank_text=True,
            resolve_entities=False,
        )
        xml = etree.XML(open(xml_file, 'rb').read(),
                        config)  # pylint: disable-msg=E1101
        for child in xml.getchildren():
            # } to prevent conflict with PhoneIdentity..
            if isinstance(child.tag, str) and child.tag.endswith('}Identity'):
                xml_dic['version'] = child.get('Version')
                xml_dic['arch'] = child.get('ProcessorArchitecture')
            elif (isinstance(child.tag, str)
                    and child.tag.endswith('Properties')):
                for sub_child in child.getchildren():
                    if sub_child.tag.endswith('}DisplayName'):
                        # TODO(Needed? Compare to existing app_name)
                        xml_dic['app_name'] = sub_child.text
                    elif sub_child.tag.endswith('}PublisherDisplayName'):
                        xml_dic['pub_name'] = sub_child.text
            elif (isinstance(child.tag, str)
                    and child.tag.endswith('}Metadata')):
                xml_dic = parse_xml_metadata(xml_dic, child)
    except Exception as exp:
        msg = 'Error Reading AppxManifest'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return xml_dic


def parse_xml_metadata(xml_dic, xml_node):
    """Return the XML Metadata."""
    for child in xml_node.getchildren():
        if child.get('Name') == 'cl.exe':
            xml_dic['compiler_version'] = child.get('Version')
        elif child.get('Name') == 'VisualStudio':
            xml_dic['visual_studio_version'] = child.get('Version')
        elif child.get('Name') == 'VisualStudioEdition':
            xml_dic['visual_studio_edition'] = child.get('Value')
        elif child.get('Name') == 'OperatingSystem':
            xml_dic['target_os'] = child.get('Version')
        elif child.get('Name') == 'Microsoft.Build.AppxPackage.dll':
            xml_dic['appx_dll_version'] = child.get('Version')
        elif child.get('Name') == 'ProjectGUID':
            xml_dic['proj_guid'] = child.get('Value')
        elif child.get('Name') == 'OptimizingToolset':
            xml_dic['opti_tool'] = child.get('Value')
        elif child.get('Name') == 'TargetRuntime':
            xml_dic['target_run'] = child.get('Value')
    return xml_dic



# -*- coding: utf_8 -*-
"""Module holding the functions for code analysis."""

import os
import logging
import tempfile
from pathlib import Path

from django.conf import settings

import yaml

from mobsf.MobSF.utils import (
    append_scan_status,
    filename_from_path,
    get_android_src_dir,
    settings_enabled,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    url_n_email_extract,
)
from mobsf.StaticAnalyzer.views.sast_engine import (
    ChoiceEngine,
    SastEngine,
)
from mobsf.MalwareAnalyzer.views.android import (
    behaviour_analysis,
)
from mobsf.StaticAnalyzer.views.android import (
    sbom_analysis,
)

logger = logging.getLogger(__name__)


def get_perm_rules(checksum, perm_rules, android_permissions):
    """Get applicable permission rules."""
    try:
        if not settings_enabled('PERM_MAPPING_ENABLED'):
            return None
        if not android_permissions:
            return None
        dynamic_rules = []
        with perm_rules.open('r') as perm_file:
            prules = yaml.load(perm_file, Loader=yaml.FullLoader)
        for p in prules:
            if p['id'] in android_permissions.keys():
                dynamic_rules.append(p)
        rules = yaml.dump(dynamic_rules)
        if rules:
            tmp = tempfile.NamedTemporaryFile(
                mode='w',
                delete=False)
            tmp.write(rules)
            tmp.close()
            return tmp
    except Exception as exp:
        msg = 'Getting Permission Rules'
        logger.error(msg)
        append_scan_status(checksum, msg, repr(exp))
    return None


def permission_transform(perm_mappings):
    """Simply permission mappings."""
    mappings = {}
    for k, v in perm_mappings.items():
        mappings[k] = v['files']
    return mappings


def code_analysis(checksum, app_dir, typ, manifest_file, android_permissions):
    """Perform the code analysis."""
    result = {
        'api': {},
        'behaviour': {},
        'perm_mappings': {},
        'findings': {},
        'niap': {},
        'urls_list': [],
        'urls': [],
        'emails': [],
        'sbom': {},
    }
    try:
        root = Path(settings.BASE_DIR) / 'StaticAnalyzer' / 'views'
        and_rules = root / 'android' / 'rules'
        code_rules = and_rules / 'android_rules.yaml'
        api_rules = and_rules / 'android_apis.yaml'
        perm_rules = and_rules / 'android_permissions.yaml'
        niap_rules = and_rules / 'android_niap.yaml'
        app_dir = Path(app_dir)
        src = get_android_src_dir(app_dir, typ).as_posix() + '/'
        skp = settings.SKIP_CLASS_PATH
        msg = f'Code Analysis Started on - {filename_from_path(src)}'
        logger.info(msg)
        append_scan_status(checksum, msg)

        options = {
            'match_rules': code_rules.as_posix(),
            'match_extensions': {'.java', '.kt'},
            'ignore_paths': skp,
        }
        sast = SastEngine(options, src)
        # Read data once and pass it to all the analysis
        file_data = sast.read_files()

        # SBOM Analysis
        result['sbom'] = sbom_analysis.sbom(app_dir, file_data)
        msg = 'Android SBOM Analysis Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # Code Analysis
        result['findings'] = sast.run_rules(
            file_data, code_rules.as_posix())
        msg = 'Android SAST Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # API Analysis
        msg = 'Android API Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        sast = SastEngine(options, src)
        result['api'] = sast.run_rules(
            file_data, api_rules.as_posix())
        msg = 'Android API Analysis Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # Permission Mapping
        rule_file = get_perm_rules(
            checksum, perm_rules, android_permissions)
        if rule_file:
            msg = 'Android Permission Mapping Started'
            logger.info(msg)
            append_scan_status(checksum, msg)
            sast = SastEngine(options, src)
            result['perm_mappings'] = permission_transform(
                sast.run_rules(file_data, rule_file.name))
            msg = 'Android Permission Mapping Completed'
            logger.info(msg)
            append_scan_status(checksum, msg)
            os.unlink(rule_file.name)

        # Behavior Analysis
        sast = SastEngine(options, src)
        result['behaviour'] = behaviour_analysis.analyze(
            checksum, sast, file_data)

        # NIAP Scan
        if settings_enabled('NIAP_ENABLED'):
            msg = 'Running NIAP Analyzer'
            logger.info(msg)
            append_scan_status(checksum, msg)
            niap_options = {
                'choice_rules': niap_rules.as_posix(),
                'alternative_path': str(manifest_file) if manifest_file else '',
                'choice_extensions': {'.java', '.xml'},
                'ignore_paths': skp,
            }
            cengine = ChoiceEngine(niap_options, src)
            file_data = cengine.read_files()
            result['niap'] = cengine.run_rules(
                file_data, niap_rules.as_posix())
            msg = 'NIAP Analysis Completed'
            logger.info(msg)
            append_scan_status(checksum, msg)

        # Extract URLs and Emails
        msg = 'Extracting Emails and URLs from Source Code'
        logger.info(msg)
        append_scan_status(checksum, msg)
        for pfile in Path(src).rglob('*'):
            if (
                (pfile.suffix in ('.java', '.kt')
                    and any(skip_path in pfile.as_posix()
                            for skip_path in skp) is False
                    and pfile.is_file())
            ):
                content = None
                try:
                    content = pfile.read_text('utf-8', 'ignore')
                    # Certain file path cannot be read in windows
                except Exception:
                    continue
                relative_java_path = pfile.as_posix().replace(src, '')
                urls, urls_nf, emails_nf = url_n_email_extract(
                    content, relative_java_path)
                result['urls_list'].extend(urls)
                result['urls'].extend(urls_nf)
                result['emails'].extend(emails_nf)
        msg = 'Email and URL Extraction Completed'
        logger.info(msg)
        append_scan_status(checksum, msg)
    except Exception as exp:
        msg = 'Failed to perform code analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return result



# -*- coding: utf_8 -*-
"""Module holding the functions for converting."""

import glob
import logging
import os
import platform
import shutil
import subprocess
import threading
import stat
from pathlib import Path
from tempfile import gettempdir

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    filename_from_path,
    find_java_binary,
    is_file_exists,
    settings_enabled,
)


logger = logging.getLogger(__name__)


def get_dex_files(app_dir):
    """Get all Dex Files for analysis."""
    glob_pattern = app_dir + '*.dex'
    return glob.glob(glob_pattern)


def dex_2_smali(checksum, app_dir, tools_dir):
    """Run dex2smali."""
    try:
        if not settings_enabled('DEX2SMALI_ENABLED'):
            return
        msg = 'Converting DEX to Smali'
        logger.info(msg)
        append_scan_status(checksum, msg)
        dexes = get_dex_files(app_dir)
        for dex_path in dexes:
            try:
                logger.info('Converting %s to Smali Code',
                            filename_from_path(dex_path))
                if (len(settings.BACKSMALI_BINARY) > 0
                        and is_file_exists(settings.BACKSMALI_BINARY)):
                    bs_path = settings.BACKSMALI_BINARY
                else:
                    bs_path = os.path.join(tools_dir, 'baksmali-3.0.8-dev-fat.jar')
                output = os.path.join(app_dir, 'smali_source/')
                smali = [
                    find_java_binary(),
                    '-jar',
                    bs_path,
                    'd',
                    dex_path,
                    '-o',
                    output,
                ]
                trd = threading.Thread(target=subprocess.call, args=(smali,))
                trd.daemon = True
                trd.start()
            except Exception:
                # Fixes a bug #2014
                pass
    except Exception as exp:
        msg = 'Failed to convert DEX to Smali'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))


def apk_2_java(checksum, app_path, app_dir, dwd_tools_dir):
    """Run JADX to decompile APK or all DEX files to Java source code."""
    try:
        jadx_version = '1.5.0'
        jadx_base_path = Path(dwd_tools_dir) / 'jadx' / f'jadx-{jadx_version}' / 'bin'
        output_dir = Path(app_dir) / 'java_source'

        msg = 'Decompiling APK to Java with JADX'
        logger.info(msg)
        append_scan_status(checksum, msg)

        # Clean output directory if it exists
        if output_dir.exists():
            shutil.rmtree(output_dir, ignore_errors=True)

        # Determine JADX executable path
        if (len(settings.JADX_BINARY) > 0
                and is_file_exists(settings.JADX_BINARY)):
            jadx = Path(settings.JADX_BINARY)
        elif platform.system() == 'Windows':
            jadx = jadx_base_path / 'jadx.bat'
        else:
            jadx = jadx_base_path / 'jadx'

        # Ensure JADX has execute permissions
        if not os.access(str(jadx), os.X_OK):
            os.chmod(str(jadx), stat.S_IEXEC)

        # Prepare the base arguments for JADX
        def run_jadx(arguments):
            """Run JADX command with the specified arguments."""
            with open(os.devnull, 'w') as fnull:
                return subprocess.run(
                    arguments,
                    stdout=fnull,
                    stderr=subprocess.STDOUT,
                    timeout=settings.JADX_TIMEOUT)

        # First attempt to decompile APK
        args = [
            str(jadx), '-ds', str(output_dir),
            '-q', '-r', '--show-bad-code', app_path]
        result = run_jadx(args)
        if result.returncode == 0:
            return  # Success

        # If APK decompilation fails, attempt to decompile all DEX files recursively
        msg = 'Decompiling with JADX failed, attempting on all DEX files'
        logger.warning(msg)
        append_scan_status(checksum, msg)

        dex_files = Path(app_path).parent.rglob('*.dex')
        decompile_failed = False

        for dex_file in dex_files:
            msg = f'Decompiling {dex_file.name} with JADX'
            logger.info(msg)
            append_scan_status(checksum, msg)

            # Update argument to point to the current DEX file
            args[-1] = str(dex_file)
            result_dex = run_jadx(args)

            if result_dex.returncode != 0:
                decompile_failed = True
                msg = f'Decompiling with JADX failed for {dex_file.name}'
                logger.error(msg)
                append_scan_status(checksum, msg)

        if decompile_failed:
            msg = 'Some DEX files failed to decompile'
            logger.error(msg)
            append_scan_status(checksum, msg)

    except subprocess.TimeoutExpired as exp:
        msg = 'Decompiling with JADX timed out'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
    except Exception as exp:
        msg = 'Decompiling with JADX failed'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))


def run_apktool(app_path, app_dir, tools_dir):
    """Get readable AndroidManifest.xml from APK."""
    try:
        if (len(settings.APKTOOL_BINARY) > 0
                and Path(settings.APKTOOL_BINARY).exists()):
            apktool_path = Path(settings.APKTOOL_BINARY)
        else:
            apktool_path = tools_dir / 'apktool_2.10.0.jar'

        # Prepare output directory and manifest file paths
        output_dir = app_dir / 'apktool_out'
        # Run apktool to extract AndroidManifest.xml
        args = [find_java_binary(),
                '-jar',
                '-Djdk.util.zip.disableZip64ExtraFieldValidation=true',
                str(apktool_path),
                '--match-original',
                '--frame-path',
                gettempdir(),
                '-f', '-s', 'd',
                str(app_path),
                '-o',
                str(output_dir)]
        logger.info('Converting AXML to XML with apktool')
        with open(os.devnull, 'w') as fnull:
            subprocess.run(
                args,
                stdout=fnull,
                stderr=subprocess.STDOUT,
                timeout=settings.JADX_TIMEOUT)
    except Exception:
        logger.warning('apktool failed to extract AndroidManifest.xml')



"""Firebase Analysis."""
import logging
from urllib.parse import urlparse

from mobsf.MobSF.utils import (
    append_scan_status,
    upstream_proxy,
)
from mobsf.MobSF.security import valid_host

import requests


logger = logging.getLogger(__name__)
HIGH = 'high'
WARNING = 'warning'
INFO = 'info'
SECURE = 'secure'
FIREBASE_FINDINGS = {
    'firebase_db_open': {
        'title': 'Open Firebase database',
        'severity': HIGH,
        'description': (
            'The Firebase database at %s is exposed'
            ' to internet without any authentication'),
    },
    'firebase_db_exists': {
        'title': 'App talks to a Firebase database',
        'severity': INFO,
        'description': 'The app talks to Firebase database at %s',
    },
    'firebase_db_check_failed': {
        'title': 'Firebase DB check failed',
        'severity': INFO,
        'description': (
            'Failed to check Firebase DB URL. Error: %s'),
    },
    'firebase_remote_config_enabled': {
        'title': 'Firebase Remote Config enabled',
        'severity': WARNING,
        'description': (
            'The Firebase Remote Config at %s is enabled.'
            ' Ensure that the configurations are not sensitive.'
            ' This is indicated by the response: %s'),
    },
    'firebase_remote_config_disabled': {
        'title': 'Firebase Remote Config disabled',
        'severity': SECURE,
        'description': (
            'Firebase Remote Config is disabled for %s.'
            ' This is indicated by the response: %s'),
    },
    'firebase_remote_config_failed': {
        'title': 'Firebase Remote Config check failed',
        'severity': INFO,
        'description': (
            'Failed to check for Firebase Remote Config.'
            ' Please verify this manually. Error: %s'),
    },
}


def firebase_analysis(checksum, code_an_dic):
    """Firebase Analysis."""
    findings = []
    logger.info('Starting Firebase Analysis')
    # Check for Firebase Database
    db_finds = firebase_db_check(
        checksum, code_an_dic)
    if db_finds:
        findings.extend(db_finds)
    # Check for Firebase Remote Config
    config_finds = firebase_remote_config(
        checksum, code_an_dic)
    if config_finds:
        findings.extend(config_finds)
    return findings


def open_firebase(checksum, url):
    # Detect Open Firebase Database
    try:
        if not valid_host(url):
            logger.warning('Invalid Host: %s', url)
            return url, False
        purl = urlparse(url)
        if not purl.netloc.lower().endswith('.firebaseio.com'):
            logger.warning('Invalid Firebase URL')
            return url, False
        base_url = f'{purl.scheme}://{purl.netloc}/.json'
        proxies, verify = upstream_proxy('https')
        headers = {
            'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1)'
                           ' AppleWebKit/537.36 (KHTML, like Gecko) '
                           'Chrome/39.0.2171.95 Safari/537.36')}
        resp = requests.get(
            base_url,
            timeout=5,
            headers=headers,
            proxies=proxies,
            verify=verify,
            allow_redirects=False)
        if resp.status_code == 200:
            return base_url, True
    except Exception as exp:
        msg = 'Open Firebase DB detection failed'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
    return url, False


def firebase_db_check(checksum, code_an_dic):
    logger.info('Looking for Firebase URL(s)')
    findings = []
    try:
        urls = list(set(code_an_dic['urls_list']))
        for url in urls:
            if '.firebaseio.com' not in url:
                continue
            returl, is_open = open_firebase(checksum, url)
            if is_open:
                rule = FIREBASE_FINDINGS['firebase_db_open']
                findings.append({
                    'title': rule['title'],
                    'severity': rule['severity'],
                    'description': rule['description'] % returl,
                })
            else:
                rule = FIREBASE_FINDINGS['firebase_db_exists']
                findings.append({
                    'title': rule['title'],
                    'severity': rule['severity'],
                    'description': rule['description'] % returl,
                })
    except Exception as exp:
        msg = 'Failed to check for Firebase DB URL'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
        rule = FIREBASE_FINDINGS['firebase_db_check_failed']
        findings.append({
            'title': rule['title'],
            'severity': rule['severity'],
            'description': rule['description'] % repr(exp),
        })
    return findings


def firebase_remote_config(checksum, code_an_dic):
    """Check for Firebase Remote Config."""
    url = None
    findings = []
    try:
        creds = code_an_dic.get('firebase_creds')
        if not creds:
            return None
        google_api_key = creds.get('google_api_key')
        google_app_id = creds.get('google_app_id')

        if not (google_app_id and google_api_key):
            return None

        project_id = google_app_id.split(':')[1]
        if not project_id.isnumeric():
            return None

        # https://docs.emergetools.com/docs/firebase-remote-config-api-exposed
        # https://blog.deesee.xyz/android/automation/2019/08/03/
        # firebase-remote-config-dump.html
        logger.info('Checking for Firebase Remote Config')
        url = (f'https://firebaseremoteconfig.googleapis.com/v1/projects/'
               f'{project_id}/namespaces/firebase:fetch?key={google_api_key}')

        body = {
            'appId': google_app_id,
            'appInstanceId': 'required_but_unused_value',
        }

        proxies, verify = upstream_proxy('https')
        response = requests.post(
            url,
            timeout=5,
            json=body,
            proxies=proxies,
            verify=verify,
            allow_redirects=False)

        if response.status_code == 200:
            resp = response.json()
            if resp.get('state') == 'NO_TEMPLATE':
                rule = FIREBASE_FINDINGS['firebase_remote_config_disabled']
                findings.append({
                    'title': rule['title'],
                    'severity': rule['severity'],
                    'description': rule['description'] % (url, resp),
                })
            else:
                rule = FIREBASE_FINDINGS['firebase_remote_config_enabled']
                findings.append({
                    'title': rule['title'],
                    'severity': rule['severity'],
                    'description': rule['description'] % (url, resp),
                })
        else:
            rule = FIREBASE_FINDINGS['firebase_remote_config_disabled']
            response_msg = f'The response code is {response.status_code}'
            findings.append({
                'title': rule['title'],
                'severity': rule['severity'],
                'description': rule['description'] % (url, response_msg),
            })
    except Exception as exp:
        msg = 'Failed to check for Firebase Remote Config'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))

        rule = FIREBASE_FINDINGS['firebase_remote_config_failed']
        findings.append({
            'title': rule['title'],
            'severity': rule['severity'],
            'description': rule['description'] % repr(exp),
        })
    return findings



# -*- coding: utf_8 -*-
"""Connect to iOS device using SSH over USB or WiFi."""

import logging
import shutil
import socket
import subprocess
import threading
import time

import paramiko

from mobsf.DynamicAnalyzer.views.ios.helpers import (
    get_ios_model_mapping,
)

logger = logging.getLogger(__name__)


class IOSConnector:
    """Connect to iOS device using SSH over USB or WiFi."""

    def __init__(self):
        """Initialize the IOSConnector."""
        self.ssh_client = None
        self.connection_type = None
        self.device_info = {}
        self.host = None
        self.port = None
        self.username = None
        self.password = None
        self.iproxy_procs = []  # Popen handles for iproxy subprocesses

    def _ssh_execute_command(self, command, timeout=30):
        """Execute a command on SSH client."""
        if not self.ssh_client:
            return None, None, None

        # Shared variables for thread communication
        result = {'output': '', 'error': '', 'exit_status':
                  - 1, 'exception': None, 'channel': None}
        command_completed = threading.Event()

        def execute_command():
            """Execute command in separate thread."""
            try:
                # Execute command
                _, stdout, stderr = self.ssh_client.exec_command(command)
                channel = stdout.channel
                result['channel'] = channel

                # Read output with partial capture
                output_parts = []
                error_parts = []

                # Read stdout in chunks to capture partial output
                while True:
                    try:
                        chunk = stdout.read(1024).decode('utf-8', errors='ignore')
                        if not chunk:
                            break
                        output_parts.append(chunk)
                    except Exception:
                        break

                # Read stderr in chunks to capture partial output
                while True:
                    try:
                        chunk = stderr.read(1024).decode('utf-8', errors='ignore')
                        if not chunk:
                            break
                        error_parts.append(chunk)
                    except Exception:
                        break

                result['output'] = ''.join(output_parts).strip()
                result['error'] = ''.join(error_parts).strip()

                # Try to get exit status if available
                try:
                    result['exit_status'] = channel.recv_exit_status()
                except Exception:
                    result['exit_status'] = -1

            except Exception as e:
                result['exception'] = e
            finally:
                command_completed.set()

        # Start command execution in separate thread
        command_thread = threading.Thread(target=execute_command, daemon=True)
        command_thread.start()

        # Wait for command to complete or timeout
        if command_completed.wait(timeout=timeout):
            if result['exception']:
                logger.error('Command execution failed: %s', str(result['exception']))
                return result['output'], result['error'], -1
            return result['output'], result['error'], result['exit_status']
        else:
            if 'oslog' not in command:
                logger.warning(
                    'Command execution timed out after %d seconds: %s',
                    timeout, command)

            # Try to close the channel if it exists
            if result['channel']:
                try:
                    result['channel'].close()
                except Exception:
                    pass

            # Return any partial output that was captured
            return result['output'], result['error'], -1

    def connect_usb(self, device_id, username, password, port=2222):
        """Connect to iOS device using USB (via usbmuxd/iproxy)."""
        try:
            logger.info('Attempting USB connection to iOS device via SSH')

            # Check if usbmuxd/iproxy is available
            if not self._check_usbmuxd():
                raise Exception(
                    'usbmuxd/iproxy not available. Install libimobiledevice.')

            # Find USB connected iOS devices
            devices = self.get_usb_devices()
            if not devices:
                raise Exception('No USB connected iOS devices found')

            # List all devices
            logger.info('Available USB devices:')
            for device in devices:
                logger.info(
                    'Device:\n'
                    '  ID:      %s\n'
                    '  Name:    %s\n'
                    '  Model:   %s\n'
                    '  Version: %s\n'
                    '  Serial:  %s',
                    device.get('id', 'Unknown'),
                    device.get('name', 'Unknown'),
                    device.get('model', 'Unknown'),
                    device.get('version', 'Unknown'),
                    device.get('serial', 'Unknown'),
                )

            # Use specified device or first available
            if device_id:
                device = next((d for d in devices if d['id'] == device_id), None)
                if not device:
                    raise Exception('Device %s not found' % device_id)
            else:
                device = devices[0]

            logger.info(
                'Selected device:\n'
                '  ID:      %s\n'
                '  Name:    %s\n'
                '  Model:   %s\n'
                '  Version: %s\n'
                '  Serial:  %s',
                device.get('id', 'Unknown'),
                device.get('name', 'Unknown'),
                device.get('model', 'Unknown'),
                device.get('version', 'Unknown'),
                device.get('serial', 'Unknown'),
            )

            # Setup port forwarding using iproxy
            self._setup_usb_port_forward(device['id'], port)

            # Prepare device info
            device_info = {
                'id': device['id'],
                'name': device['name'],
                'type': 'usb',
                'connection': f'localhost:{port}',
            }

            # Establish SSH connection
            self._establish_ssh_connection(
                'localhost', port, username, password, 'usb', device_info)

            logger.info('Successfully connected to %s via USB SSH', device['name'])
            return True

        except Exception as e:
            logger.error('USB connection failed: %s', str(e))
            return False

    def connect_wifi(self, ip_address, username, password, port=22):
        """Connect to iOS device using WiFi SSH."""
        try:
            if port not in range(1, 65536):
                raise Exception('Invalid port number')
            logger.info('Attempting WiFi SSH connection to %s:%s', ip_address, port)

            # Prepare device info
            device_info = {
                'ip': ip_address,
                'port': port,
                'type': 'wifi',
                'connection': f'{ip_address}:{port}',
            }

            # Establish SSH connection
            self._establish_ssh_connection(
                ip_address, port, username, password, 'wifi', device_info)
            return True
        except Exception as e:
            logger.error('WiFi connection failed: %s', str(e))
            return False

    def disconnect(self):
        """Disconnect from iOS device."""
        try:
            if self.ssh_client:
                self.ssh_client.close()
                self.ssh_client = None

            # Terminate only the iproxy processes started by this instance
            for proc in self.iproxy_procs:
                try:
                    proc.terminate()
                except Exception:
                    pass
            self.iproxy_procs = []
            logger.info('Disconnected from iOS device')
        except Exception as e:
            logger.error('Error during disconnect: %s', str(e))
        finally:
            if hasattr(self, 'ssh_client') and self.ssh_client:
                try:
                    self.ssh_client.close()
                except Exception:
                    pass
            self.ssh_client = None
            self.connection_type = None
            self.device_info = {}
            self.host = None
            self.port = None
            self.username = None
            self.password = None

    def get_usb_devices(self):
        """Get list of USB connected iOS devices."""
        try:
            logger.info('Getting iOS devices connected via USB')
            if not shutil.which('idevice_id'):
                logger.warning(
                    'libimobiledevice is not installed. '
                    'Install it with: brew install libimobiledevice')
                return []
            # Use idevice_id to list devices
            result = subprocess.run(['idevice_id', '-l'],
                                    capture_output=True, text=True)
            if result.returncode != 0:
                return []

            devices = []
            model_mapping = get_ios_model_mapping()
            for device_id in result.stdout.strip().split('\n'):
                if device_id:
                    device_info = {}
                    # Get device details
                    output = subprocess.run(['ideviceinfo', '-u', device_id],
                                            capture_output=True, text=True)
                    if output.returncode != 0:
                        continue
                    for line in output.stdout.splitlines():
                        if ': ' in line:
                            key, value = line.split(': ', 1)
                            device_info[key.strip()] = value.strip()

                    devices.append({
                        'id': device_id,
                        'name': device_info.get('DeviceName'),
                        'model': model_mapping.get(
                            device_info.get('ProductType'),
                            device_info.get('ProductType'),
                        ),
                        'version': device_info.get('ProductVersion'),
                        'serial': device_info.get('SerialNumber'),
                    })

            return devices
        except Exception as e:
            logger.error('Failed to get USB devices: %s', str(e))
            return []

    def _create_ssh_connection(self, host, port, username, password, timeout=10):
        """Create and connect an SSH client."""
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(
            hostname=host,
            port=port,
            username=username,
            password=password,
            timeout=timeout,
            allow_agent=False,
            look_for_keys=False,
        )
        return ssh

    def _test_connection_ssh(
            self, host, port, username, password, timeout=10):
        """Test SSH connection to iOS device."""
        try:
            logger.info('Testing SSH connection to %s:%s', host, port)

            # Create SSH client
            ssh = self._create_ssh_connection(host, port, username, password, timeout)

            # Temporarily set self.ssh_client to test client
            self.ssh_client = ssh

            # Test basic command execution - check user identity
            output, _, _ = self._ssh_execute_command('id', timeout)

            # Check if we have root access
            if 'uid=0(root)' in output:
                logger.info('Verified iOS device is Jailbroken')
                return True

            logger.warning('Not connected as root: %s', output)
            ssh.close()
            self.ssh_client = None
            return False

        except Exception as e:
            logger.error('SSH connection test failed: %s', str(e))
            if self.ssh_client:
                try:
                    self.ssh_client.close()
                except Exception:
                    pass
                self.ssh_client = None
            return False

    def _establish_ssh_connection(
            self, host, port, username, password, connection_type, device_info):
        """Establish SSH connection and set up instance variables."""
        # Test SSH connection first
        if not self._test_connection_ssh(host, port, username, password):
            raise Exception('SSH connection test failed')

        # Establish SSH connection
        if not self.ssh_client:
            self.ssh_client = self._create_ssh_connection(
                host, port, username, password)

        # Set up instance variables
        self.connection_type = connection_type
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.device_info = device_info

        logger.info('Successfully established SSH connection to %s:%s', host, port)
        return True

    def _check_usbmuxd(self):
        """Check if usbmuxd/iproxy is available."""
        try:
            result = subprocess.run(['which', 'iproxy'],
                                    capture_output=True, text=True)
            return result.returncode == 0
        except Exception:
            return False

    def _setup_usb_port_forward(self, device_id, port):
        """Setup port forwarding for USB device."""
        try:
            # Test if ports are open with separate sockets
            def test_port(port_to_test):
                try:
                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    sock.settimeout(5)
                    result = sock.connect_ex(('localhost', port_to_test))
                    sock.close()
                    return result == 0
                except Exception:
                    return False

            # Check if port forwarding already exists
            ssh_port_open = test_port(port)
            frida_port_open = test_port(37042)

            if ssh_port_open and frida_port_open:
                logger.info(
                    'Port forwarding already exists: '
                    'localhost:%s -> device:22, localhost:37042 -> device:27042',
                    port,
                )
                return True

            # Port forwarding doesn't exist, so set it up
            logger.info('Setting up new port forwarding for device %s', device_id)
            # Kill any existing iproxy processes for this device (in case they're stuck)
            subprocess.run(['pkill', '-f', f'iproxy.*{device_id}'],
                           capture_output=True)
            time.sleep(1)

            # Start iproxy for port forwarding; store handles for clean teardown
            cmds = [['iproxy', str(port), '22', '-u', device_id],
                    ['iproxy', '37042', '27042', '-u', device_id]]
            for cmd in cmds:
                logger.info('Starting iproxy: %s', ' '.join(cmd))
                proc = subprocess.Popen(
                    cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                self.iproxy_procs.append(proc)

            # Wait for port forwarding to be ready
            logger.info('Waiting for port forwarding to be ready...')
            time.sleep(2)  # Give iproxy time to establish connections

            # Test if ports are now open
            ssh_port_open = test_port(port)
            frida_port_open = test_port(37042)

            if not ssh_port_open or not frida_port_open:
                for proc in self.iproxy_procs:
                    try:
                        proc.terminate()
                    except Exception:
                        pass
                self.iproxy_procs = []
                failed_ports = []
                if not ssh_port_open:
                    failed_ports.append(f'SSH port {port}')
                if not frida_port_open:
                    failed_ports.append('Frida port 37042')
                raise Exception(
                    f"Port forwarding failed for: {', '.join(failed_ports)}")

            logger.info(
                'Port forwarding established: '
                'localhost:%s -> device:22, localhost:37042 -> device:27042',
                port,
            )
            return True

        except Exception as e:
            logger.error('Failed to setup USB port forwarding: %s', str(e))
            return False



# -*- coding: utf_8 -*-
"""iOS Device Environment for Dynamic Analysis."""

import io
import logging
import plistlib
import time
from pathlib import Path

from django.conf import settings

from frida import __version__ as frida_version

from mobsf.DynamicAnalyzer.views.common.frida.server_update import (
    FridaServerUpdater,
)

logger = logging.getLogger(__name__)


class IOSEnvironment:
    """iOS Device Environment for Dynamic Analysis."""

    def __init__(self, ios_device):
        """Initialize iOS Environment with device operations.

        Args:
            ios_device: IOSDevice instance
        """
        self.ios_device = ios_device
        self.connector = ios_device.connector
        self.frida_server_path = None
        self.frida_process = None
        self.frida_plist_path = None
        self.platform = None
        self.platform_architecture = self.ios_device.get_platform_architecture()

    def _find_frida_server_plist(self):
        """Find Frida server plist file on iOS device."""
        if self.frida_plist_path:
            return self.frida_plist_path

        try:
            # Common locations where Frida server plist might be installed
            plist_locations = [
                '/Library/LaunchDaemons/re.frida.server.plist',
                '/var/jb/Library/LaunchDaemons/re.frida.server.plist',
                '/System/Library/LaunchDaemons/re.frida.server.plist',
            ]

            for plist_path in plist_locations:
                output, _, exit_code = self.ios_device.execute_command(
                    f'ls -la {plist_path}',
                )

                if exit_code == 0 and 'No such file' not in output:
                    self.frida_plist_path = plist_path
                    return plist_path

            # If not found in common locations, search for it
            logger.info('Searching for Frida server plist files...')
            output, _, exit_code = self.ios_device.execute_command(
                'find / -name "*frida*server*.plist" 2>/dev/null',
            )

            if exit_code == 0 and output.strip():
                plist_files = output.strip().split('\n')
                for plist_file in plist_files:
                    if plist_file.strip():
                        self.frida_plist_path = plist_file.strip()
                        return plist_file.strip()

            logger.warning('No Frida server plist file found')
            return None

        except Exception:
            logger.exception('[ERROR] Finding Frida server plist')
            return None

    def _install_frida_server_on_device(self):
        """Install Frida server on iOS device."""
        try:
            logger.info('Installing Frida Server v%s on iOS device', frida_version)

            # Upload the binary to the device
            success = self.ios_device.upload_file(
                self.frida_server_path,
                '/tmp/fd.deb',
            )

            if not success:
                raise Exception(
                    'Failed to upload Frida server v%s binary to device', frida_version)

            success = self.ios_device.install_deb('/tmp/fd.deb')
            if not success:
                raise Exception(
                    'Failed to install Frida server v%s on device', frida_version)
            return True

        except Exception:
            logger.exception(
                '[ERROR] Installing Frida server v%s on iOS device', frida_version)
        return False

    def _modify_frida_plist_for_wifi(self):
        """Modify Frida server plist to enable WiFi access using plistlib."""
        try:
            self._find_frida_server_plist()
            if not self.frida_plist_path:
                logger.error('No Frida plist path available')
                return False

            logger.info('Modifying Frida plist for WiFi access')
            # Read as raw bytes — handles both XML and binary plist formats.
            # read_file() would fail on binary plists due to UTF-8 decoding.
            plist_bytes = self.ios_device.read_binary_file(self.frida_plist_path)
            if not plist_bytes:
                raise Exception('Failed to read Frida plist')

            # Quick check for already-patched XML plist (avoids re-parsing)
            if (b'<string>-l</string>' in plist_bytes
                    and b'<string>0.0.0.0</string>' in plist_bytes):
                logger.info('WiFi arguments already present in plist')
                return True

            # plistlib.loads() handles both XML and binary plist formats
            plist = plistlib.loads(plist_bytes)
            args = plist.get('ProgramArguments', [])
            if '-l' not in args:
                args.append('-l')
            if '0.0.0.0' not in args:
                args.append('0.0.0.0')
            plist['ProgramArguments'] = args
            logger.info('Added -l 0.0.0.0 to ProgramArguments')

            # Serialize back to XML and upload in-memory
            updated_bytes = plistlib.dumps(plist, fmt=plistlib.FMT_XML)
            plist_name = Path(self.frida_plist_path).name
            plist_dir = str(Path(self.frida_plist_path).parent) + '/'
            self.ios_device.upload_file_object(
                io.BytesIO(updated_bytes), plist_name, plist_dir)

            logger.info('Successfully modified Frida plist for WiFi access')
            return True

        except Exception:
            logger.exception('[ERROR] Modifying Frida plist for WiFi access')
        return False

    def _enable_frida_on_wifi(self):
        """Enable Frida on WiFi."""
        try:
            logger.info('Enabling Frida Server v%s on 0.0.0.0', frida_version)
            self._find_frida_server_plist()
            if not self.frida_plist_path:
                raise ValueError('Frida server plist not found')

            # Modify the plist to add WiFi arguments
            if not self._modify_frida_plist_for_wifi():
                raise ValueError('Failed to modify plist for WiFi access')

            # Reload the modified plist
            self.start_frida_server()
            logger.info(
                'Successfully started Frida Server v%s on (0.0.0.0)', frida_version)
            return True

        except Exception:
            logger.exception(
                '[ERROR] Enabling Frida Server v%s on 0.0.0.0', frida_version)
        return False

    def setup_or_start_frida(self):
        """Setup and start Frida server for iOS device."""
        try:
            # Check if setup is already done
            if self.ios_device.read_file('/tmp/frida_setup_done') == frida_version:
                self.start_frida_server()
                logger.info(
                    'Successfully started Frida Server v%s on (0.0.0.0)', frida_version)
                return True

            # Get iOS platform architecture
            if not self.platform_architecture:
                raise ValueError('Failed to determine iOS platform architecture')

            logger.info('iOS platform architecture identified as %s',
                        self.platform_architecture)

            # Update/download Frida server
            platform = 'iphoneos'
            frida_bin = (
                f'frida_{frida_version}_{platform}-{self.platform_architecture}.deb')
            self.frida_server_path = Path(settings.DWD_DIR) / frida_bin
            if not self.frida_server_path.is_file():
                # Download Frida server binary if not found
                updater = FridaServerUpdater(platform, frida_version)
                success = updater.update_frida_server(
                    self.platform_architecture,
                )
                if not success:
                    msg = (
                        'Cannot download frida binary for iOS. You will need'
                        f' {frida_bin} in {settings.DWD_DIR}'
                        ' for Dynamic Analysis to work')
                    raise ValueError(msg)

            # install frida server on device
            self._install_frida_server_on_device()

            # Enable and start Frida on WiFi
            self._enable_frida_on_wifi()

            # Install oslog on device
            self.install_oslog()

            # Create a file to indicate that frida setup is done
            self.ios_device.write_file('/tmp/frida_setup_done', frida_version)
            return True
        except Exception:
            logger.exception('[ERROR] Setting up Frida for iOS')
            return False

    def start_frida_server(self):
        """Start Frida Server on iOS device using launchctl."""
        try:
            self.stop_frida_server()
            # Load and start the Frida server using launchctl
            output, _, exit_code = self.ios_device.execute_command(
                f'launchctl load {self.frida_plist_path}',
            )

            if exit_code != 0:
                raise ValueError('Failed to load Frida server plist')

            # Wait for the server to start
            logger.info('Waiting for Frida server to start...')

            # Verify the server is running
            output, _, exit_code = self.ios_device.execute_command(
                'ps aux | grep frida-server | grep -v grep',
            )

            if exit_code != 0:
                # Attempt to start the server using the binary
                _, error, exit_code = self.ios_device.execute_command(
                    'frida-server -l 0.0.0.0 &',
                )
                if exit_code != 0:
                    raise ValueError(
                        f'Failed to start Frida server using binary: {error}',
                    )
                output, _, exit_code = self.ios_device.execute_command(
                    'ps aux | grep frida-server | grep -v grep',
                )
            time.sleep(1)
            if exit_code == 0 and output.strip():
                logger.info('Frida Server started successfully')
                return True
            else:
                raise Exception('Frida Server failed to start')

        except Exception:
            logger.exception('[ERROR] Running Frida server on iOS device')
        return False

    def stop_frida_server(self):
        """Stop Frida server on iOS device using launchctl."""
        try:
            self._find_frida_server_plist()
            if not self.frida_plist_path:
                raise Exception('Frida server plist not found')

            # Unload the plist using launchctl
            output, error, exit_code = self.ios_device.execute_command(
                f'launchctl unload {self.frida_plist_path}',
            )

            if exit_code == 0:
                logger.info('Successfully stopped Frida server')
            else:
                logger.warning('Failed to stop Frida server: %s', error)

            # Also kill any remaining Frida server processes as fallback
            _, error, exit_code = self.ios_device.execute_command(
                'pkill -f frida-server',
            )

            if exit_code == 0:
                logger.info('Killed remaining Frida server processes')

            return True
        except Exception:
            logger.exception('[ERROR] Stopping Frida server')
        return False

    def install_oslog(self):
        """Install oslog on iOS device."""
        try:
            logger.info('Installing oslog on iOS device')
            if not self.platform_architecture:
                raise ValueError('Failed to determine iOS platform architecture')
            tools_dir = Path(settings.TOOLS_DIR) / 'ios' / 'oslog'
            if self.platform_architecture == 'arm64':
                oslog_bin = 'oslog_0.0.4.3_iphoneos-arm64.deb'
            else:
                oslog_bin = 'oslog_0.0.1-8_iphoneos-arm.deb'
            deb_file = tools_dir / oslog_bin
            if not deb_file.exists():
                raise ValueError('oslog deb file does not exist: %s', deb_file)
            remote_path = '/tmp/oslog.deb'
            self.ios_device.upload_file(deb_file, remote_path)
            if not self.ios_device.install_deb(remote_path):
                raise ValueError('Failed to install oslog on device')
            return True
        except Exception:
            logger.exception('[ERROR] Installing oslog on iOS device')
        return False



# -*- coding: utf_8 -*-
"""MobSF REST API V 1 - iOS Jailbroken Device."""

from django.views.decorators.csrf import csrf_exempt

from mobsf.MobSF.views.helpers import request_method
from mobsf.MobSF.views.api.api_middleware import make_api_response
from mobsf.DynamicAnalyzer.views.ios.device import (
    dynamic_analyzer as ios_device,
    report as device_report,
)


FAILED = 'failed'
ERROR = 'error'
DEVICE_ID = 'device_id'
BUNDLE_ID = 'bundle_id'


@request_method(['POST'])
@csrf_exempt
def api_device_dynamic_analysis(request):
    """POST - iOS Device Dynamic Analysis Entrypoint."""
    if DEVICE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.dynamic_analysis_device(request, True)
    if ERROR in resp:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_get(request):
    """POST - Get iOS Device Details and List Apps."""
    if DEVICE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.get_ios_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_install_ipa(request):
    """POST - Install IPA on iOS Device."""
    params = {DEVICE_ID, 'checksum'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.install_ipa_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_dynamic_analyzer(request):
    """POST - iOS Device Dynamic Analyzer."""
    params = {DEVICE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.dynamic_analyzer_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_file_upload(request):
    """POST - Upload file to iOS Device."""
    if DEVICE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    if 'file' not in request.FILES:
        return make_api_response(
            {'error': 'Missing File'}, 422)
    resp = ios_device.upload_file_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_ssh_execute(request):
    """POST - Execute commands on iOS Device over SSH."""
    params = {DEVICE_ID, 'cmd'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.ssh_execute_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_instrument(request):
    """POST - Frida Instrument on iOS Device."""
    params = {
        DEVICE_ID,
        BUNDLE_ID,
        'default_hooks',
        'dump_hooks',
        'auxiliary_hooks',
        'frida_code'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.ios_instrument_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_system_logs(request):
    """POST - Get system logs from iOS Device."""
    if DEVICE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.system_logs_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_ps(request):
    """POST - Get running processes on iOS Device."""
    if DEVICE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = ios_device.ps_device(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_file_download(request):
    """POST - Download file from iOS Device."""
    params = {DEVICE_ID, 'file'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = device_report.download_file_device(request, True)
    if resp.get('Content-Disposition'):
        # file http response
        return resp
    elif resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_api_monitor(request):
    """POST - iOS Device API Monitor."""
    if 'checksum' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = device_report.ios_api_monitor(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_download_app_data(request):
    """POST - Download Application Data from iOS Device."""
    params = {DEVICE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = device_report.download_data_device(
        request,
        request.POST[BUNDLE_ID],
        True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_report_json(request):
    """POST - iOS Device Dynamic Analysis Report."""
    params = {DEVICE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = device_report.view_report_device(
        request,
        request.POST[BUNDLE_ID],
        True)
    if 'error' in resp:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)



# -*- coding: utf_8 -*-
"""IPA Binary Analysis Rules."""
from libsast.standards import get_standards

HIGH = 'high'
WARNING = 'warning'
INFO = 'info'
STDS = get_standards()
IPA_RULES = [
    {
        'description': 'Binary makes use of insecure API(s)',
        'detailed_desc': (
            'The binary may contain the following insecure API(s) {}'),
        'type': 'Regex',
        'pattern': (
            rb'\b_alloca\n\b|\b_gets\n\b|\b_memcpy\n\b|\b_printf\n\b|'
            rb'\b_scanf\n\b|\b_sprintf\n\b|\b_sscanf\n\b|\b_strcat\n\b|'
            rb'\bStrCat\n\b|\b_strcpy\n\b|\bStrCpy\n\b|\b_strlen\n\b|'
            rb'\bStrLen\n\b|\b_strncat\n\b|\bStrNCat\n\b|\b_strncpy\n\b|'
            rb'\bStrNCpy\n\b|\b_strtok\n\b|\b_swprintf\n\b|\b_vsnprintf\n\b|'
            rb'\b_vsprintf\b|\b_vswprintf\b|\b_wcscat\b|\b_wcscpy\b|'
            rb'\b_wcslen\n\b|\b_wcsncat\n\b|\b_wcsncpy\n\b|\b_wcstok\n\b|'
            rb'\b_wmemcpy\n\b|\b_fopen\n\b|\b_chmod\n\b|\b_chown\n\b|'
            rb'\b_stat\n\b|\b_mktemp\n\b'),
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 6,
        'cwe': STDS['cwe']['cwe-676'],
        'owasp-mobile': STDS['owasp-mobile']['m7'],
        'masvs': STDS['masvs']['code-8'],
    },
    {
        'description': 'Binary makes use of some weak Crypto API(s)',
        'detailed_desc': (
            'The binary may use the following weak crypto API(s) {}'),
        'type': 'Regex',
        'pattern': (
            rb'\bkCCAlgorithmDES\b|'
            rb'\bkCCAlgorithm3DES\b|'
            rb'\bkCCAlgorithmRC2\b|'
            rb'\bkCCAlgorithmRC4\b|'
            rb'\bkCCOptionECBMode\b|'
            rb'\bkCCOptionCBCMode\b'),
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 3,
        'cwe': STDS['cwe']['cwe-327'],
        'owasp-mobile': STDS['owasp-mobile']['m5'],
        'masvs': STDS['masvs']['crypto-3'],
    },
    {
        'description': 'Binary makes use of the following Crypto API(s)',
        'detailed_desc': (
            'The binary may use the following crypto API(s) {}'),
        'type': 'Regex',
        'pattern': (
            rb'\bCCKeyDerivationPBKDF\b|\bCCCryptorCreate\b|\b'
            rb'CCCryptorCreateFromData\b|\b'
            rb'CCCryptorRelease\b|\bCCCryptorUpdate\b|\bCCCryptorFinal\b|\b'
            rb'CCCryptorGetOutputLength\b|\bCCCryptorReset\b|\b'
            rb'CCCryptorRef\b|\bkCCEncrypt\b|\b'
            rb'kCCDecrypt\b|\bkCCAlgorithmAES128\b|\bkCCKeySizeAES128\b|\b'
            rb'kCCKeySizeAES192\b|\b'
            rb'kCCKeySizeAES256\b|\bkCCAlgorithmCAST\b|\b'
            rb'SecCertificateGetTypeID\b|\b'
            rb'SecIdentityGetTypeID\b|\bSecKeyGetTypeID\b|\b'
            rb'SecPolicyGetTypeID\b|\b'
            rb'SecTrustGetTypeID\b|\bSecCertificateCreateWithData\b|\b'
            rb'SecCertificateCreateFromData\b|\bSecCertificateCopyData\b|\b'
            rb'SecCertificateAddToKeychain\b|\bSecCertificateGetData\b|\b'
            rb'SecCertificateCopySubjectSummary\b|\b'
            rb'SecIdentityCopyCertificate\b|\b'
            rb'SecIdentityCopyPrivateKey\b|\bSecPKCS12Import\b|\b'
            rb'SecKeyGeneratePair\b|\b'
            rb'SecKeyEncrypt\b|\bSecKeyDecrypt\b|\bSecKeyRawSign\b|\b'
            rb'SecKeyRawVerify\b|\b'
            rb'SecKeyGetBlockSize\b|\bSecPolicyCopyProperties\b|\b'
            rb'SecPolicyCreateBasicX509\b|\bSecPolicyCreateSSL\b|\b'
            rb'SecTrustCopyCustomAnchorCertificates\b|\b'
            rb'SecTrustCopyExceptions\b|\b'
            rb'SecTrustCopyProperties\b|\bSecTrustCopyPolicies\b|\b'
            rb'SecTrustCopyPublicKey\b|\bSecTrustCreateWithCertificates\b|\b'
            rb'SecTrustEvaluate\b|\bSecTrustEvaluateAsync\b|\b'
            rb'SecTrustGetCertificateCount\b|\b'
            rb'SecTrustGetCertificateAtIndex\b|\b'
            rb'SecTrustGetTrustResult\b|\bSecTrustGetVerifyTime\b|\b'
            rb'SecTrustSetAnchorCertificates\b|\b'
            rb'SecTrustSetAnchorCertificatesOnly\b|\b'
            rb'SecTrustSetExceptions\b|\bSecTrustSetPolicies\b|\b'
            rb'SecTrustSetVerifyDate\b|\bSecCertificateRef\b|\b'
            rb'SecIdentityRef\b|\bSecKeyRef\b|\bSecPolicyRef\b|\b'
            rb'SecTrustRef\b'),
        'severity': INFO,
        'input_case': 'exact',
        'cvss': 0,
        'cwe': '',
        'owasp-mobile': '',
        'masvs': '',
    },
    {
        'description': 'Binary makes use of some weak Hashing API(s)',
        'detailed_desc': (
            'The binary may use the following weak hash API(s) {}'),
        'type': 'Regex',
        'pattern': (
            rb'\bCC_MD2_Init\b|\bCC_MD2_Update\b|\b'
            rb'CC_MD2_Final\b|\bCC_MD2\b|\bMD2_Init\b|\b'
            rb'MD2_Update\b|\bMD2_Final\b|\bCC_MD4_Init\b|\b'
            rb'CC_MD4_Update\b|\bCC_MD4_Final\b|\b'
            rb'CC_MD4\b|\bMD4_Init\b|\bMD4_Update\b|\b'
            rb'MD4_Final\b|\bCC_MD5_Init\b|\bCC_MD5_Update'
            rb'\b|\bCC_MD5_Final\b|\bCC_MD5\b|\bMD5_Init\b|\b'
            rb'MD5_Update\b|\bMD5_Final\b|\bMD5Init\b|\b'
            rb'MD5Update\b|\bMD5Final\b|\bCC_SHA1_Init\b|\b'
            rb'CC_SHA1_Update\b|\b'
            rb'CC_SHA1_Final\b|\bCC_SHA1\b|\bSHA1_Init\b|\b'
            rb'SHA1_Update\b|\bSHA1_Final\b'),
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 3,
        'cwe': STDS['cwe']['cwe-327'],
        'owasp-mobile': STDS['owasp-mobile']['m5'],
        'masvs': STDS['masvs']['crypto-4'],
    },
    {
        'description': 'Binary makes use of the following Hash API(s)',
        'detailed_desc': (
            'The binary may use the following hash API(s) {}'),
        'type': 'Regex',
        'pattern': (
            rb'\bCC_SHA224_Init\b|\bCC_SHA224_Update\b|\b'
            rb'CC_SHA224_Final\b|\bCC_SHA224\b|\b'
            rb'SHA224_Init\b|\bSHA224_Update\b|\b'
            rb'SHA224_Final\b|\bCC_SHA256_Init\b|\b'
            rb'CC_SHA256_Update\b|\bCC_SHA256_Final\b|\b'
            rb'CC_SHA256\b|\bSHA256_Init\b|\b'
            rb'SHA256_Update\b|\bSHA256_Final\b|\b'
            rb'CC_SHA384_Init\b|\bCC_SHA384_Update\b|\b'
            rb'CC_SHA384_Final\b|\bCC_SHA384\b|\b'
            rb'SHA384_Init\b|\bSHA384_Update\b|\b'
            rb'SHA384_Final\b|\bCC_SHA512_Init\b|\b'
            rb'CC_SHA512_Update\b|\bCC_SHA512_Final\b|\b'
            rb'CC_SHA512\b|\bSHA512_Init\b|\b'
            rb'SHA512_Update\b|\bSHA512_Final\b'),
        'severity': INFO,
        'input_case': 'exact',
        'cvss': 0,
        'cwe': '',
        'owasp-mobile': '',
        'masvs': '',
    },
    {
        'description': 'Binary makes use of the insecure Random function(s)',
        'detailed_desc': (
            'The binary may use the following '
            'insecure Random function(s) {}'),
        'type': 'Regex',
        'pattern': rb'\b_srand\n\b|\b_random\n\b',
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 3,
        'cwe': STDS['cwe']['cwe-330'],
        'owasp-mobile': STDS['owasp-mobile']['m5'],
        'masvs': STDS['masvs']['crypto-6'],
    },
    {
        'description': 'Binary makes use of Logging function',
        'detailed_desc': (
            'The binary may use {} function for logging.'),
        'type': 'Regex',
        'pattern': rb'\b_NSLog\n\b',
        'severity': INFO,
        'input_case': 'exact',
        'cvss': 7.5,
        'cwe': STDS['cwe']['cwe-532'],
        'owasp-mobile': '',
        'masvs': STDS['masvs']['storage-3'],
    },
    {
        'description': 'Binary makes use of malloc function',
        'detailed_desc': (
            'The binary may use {} function instead of calloc'),
        'type': 'Regex',
        'pattern': rb'_malloc\n',
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 2,
        'cwe': STDS['cwe']['cwe-789'],
        'owasp-mobile': STDS['owasp-mobile']['m7'],
        'masvs': STDS['masvs']['code-8'],
    },
    {
        'description': 'Binary calls ptrace() function for anti-debugging.',
        'detailed_desc': (
            'The binary may use ptrace() function. It can be'
            ' used to detect and prevent debuggers.'
            ' Ptrace is not a public API and apps that'
            ' use non-public APIs will be rejected'
            ' from AppStore.'),
        'type': 'Regex',
        'pattern': rb'\b_ptrace\b',
        'severity': WARNING,
        'input_case': 'exact',
        'cvss': 0,
        'cwe': '',
        'owasp-mobile': STDS['owasp-mobile']['m7'],
        'masvs': STDS['masvs']['resilience-2'],
    },
    {
        'description': 'Binary uses WebView Component.',
        'detailed_desc': 'The binary may use UIWebView Component.',
        'type': 'Regex',
        'pattern': b'UIWebView',
        'severity': INFO,
        'input_case': 'exact',
        'cvss': 0,
        'cwe': '',
        'owasp-mobile': '',
        'masvs': STDS['masvs']['code-9'],
    },
]



from datetime import datetime
from enum import Enum

from django.db import models
from django.utils import timezone


class DjangoPermissions(Enum):
    SCAN = ('can_scan', 'Scan Files')
    SUPPRESS = ('can_suppress', 'Suppress Findings')
    DELETE = ('can_delete', 'Delete Scans')


P = DjangoPermissions


class RecentScansDB(models.Model):
    class Meta:
        """Meta class for RecentScansDB model."""

        permissions = (P.DELETE.value, P.SCAN.value)

    ANALYZER = models.CharField(max_length=50, default='')
    SCAN_TYPE = models.CharField(max_length=10, default='')
    FILE_NAME = models.CharField(max_length=260, default='')
    APP_NAME = models.CharField(max_length=260, default='')
    PACKAGE_NAME = models.CharField(max_length=260, default='')
    VERSION_NAME = models.CharField(max_length=50, default='')
    MD5 = models.CharField(max_length=32, default='', primary_key=True)
    TIMESTAMP = models.DateTimeField(default=datetime.now)
    SCAN_LOGS = models.TextField(default=[])


class StaticAnalyzerAndroid(models.Model):
    class Meta:
        """Meta class for StaticAnalyzerAndroid model."""

        permissions = (P.DELETE.value, P.SCAN.value)

    FILE_NAME = models.CharField(max_length=260, default='')
    APP_NAME = models.CharField(max_length=255, default='')
    APP_TYPE = models.CharField(max_length=20, default='')
    SIZE = models.CharField(max_length=50, default='')
    MD5 = models.CharField(max_length=32, default='', primary_key=True)
    SHA1 = models.CharField(max_length=40, default='')
    SHA256 = models.CharField(max_length=64, default='')
    PACKAGE_NAME = models.TextField(default='')
    MAIN_ACTIVITY = models.TextField(default='')
    EXPORTED_ACTIVITIES = models.TextField(default='')
    BROWSABLE_ACTIVITIES = models.TextField(default={})
    ACTIVITIES = models.TextField(default=[])
    RECEIVERS = models.TextField(default=[])
    PROVIDERS = models.TextField(default=[])
    SERVICES = models.TextField(default=[])
    LIBRARIES = models.TextField(default=[])
    TARGET_SDK = models.CharField(max_length=50, default='')
    MAX_SDK = models.CharField(max_length=50, default='')
    MIN_SDK = models.CharField(max_length=50, default='')
    VERSION_NAME = models.CharField(max_length=100, default='')
    VERSION_CODE = models.CharField(max_length=50, default='')
    ICON_PATH = models.TextField(default='')
    PERMISSIONS = models.TextField(default={})
    MALWARE_PERMISSIONS = models.TextField(default={})
    CERTIFICATE_ANALYSIS = models.TextField(default={})
    MANIFEST_ANALYSIS = models.TextField(default=[])
    BINARY_ANALYSIS = models.TextField(default=[])
    FILE_ANALYSIS = models.TextField(default=[])
    ANDROID_API = models.TextField(default={})
    CODE_ANALYSIS = models.TextField(default={})
    NIAP_ANALYSIS = models.TextField(default={})
    PERMISSION_MAPPING = models.TextField(default={})
    URLS = models.TextField(default=[])
    DOMAINS = models.TextField(default={})
    EMAILS = models.TextField(default=[])
    STRINGS = models.TextField(default={})
    FIREBASE_URLS = models.TextField(default=[])
    FILES = models.TextField(default=[])
    EXPORTED_COUNT = models.TextField(default={})
    APKID = models.TextField(default={})
    QUARK = models.TextField(default=[])
    TRACKERS = models.TextField(default={})
    PLAYSTORE_DETAILS = models.TextField(default={})
    NETWORK_SECURITY = models.TextField(default=[])
    SECRETS = models.TextField(default=[])
    SBOM = models.TextField(default={})


class StaticAnalyzerIOS(models.Model):
    class Meta:
        """Meta class for StaticAnalyzerIOS model."""

        permissions = (P.DELETE.value, P.SCAN.value)

    FILE_NAME = models.CharField(max_length=255, default='')
    APP_NAME = models.CharField(max_length=255, default='')
    APP_TYPE = models.CharField(max_length=20, default='')
    SIZE = models.CharField(max_length=50, default='')
    MD5 = models.CharField(max_length=32, default='', primary_key=True)
    SHA1 = models.CharField(max_length=40, default='')
    SHA256 = models.CharField(max_length=64, default='')
    BUILD = models.TextField(default='')
    APP_VERSION = models.CharField(max_length=100, default='')
    SDK_NAME = models.CharField(max_length=50, default='')
    PLATFORM = models.CharField(max_length=50, default='')
    MIN_OS_VERSION = models.CharField(max_length=50, default='')
    BUNDLE_ID = models.TextField(default='')
    BUNDLE_URL_TYPES = models.TextField(default=[])
    BUNDLE_SUPPORTED_PLATFORMS = models.CharField(max_length=50, default=[])
    ICON_PATH = models.TextField(default='')
    INFO_PLIST = models.TextField(default='')
    BINARY_INFO = models.TextField(default={})
    PERMISSIONS = models.TextField(default={})
    ATS_ANALYSIS = models.TextField(default=[])
    BINARY_ANALYSIS = models.TextField(default=[])
    MACHO_ANALYSIS = models.TextField(default={})
    DYLIB_ANALYSIS = models.TextField(default=[])
    FRAMEWORK_ANALYSIS = models.TextField(default=[])
    IOS_API = models.TextField(default={})
    CODE_ANALYSIS = models.TextField(default={})
    FILE_ANALYSIS = models.TextField(default=[])
    LIBRARIES = models.TextField(default=[])
    FILES = models.TextField(default=[])
    URLS = models.TextField(default=[])
    DOMAINS = models.TextField(default={})
    EMAILS = models.TextField(default=[])
    STRINGS = models.TextField(default=[])
    FIREBASE_URLS = models.TextField(default=[])
    APPSTORE_DETAILS = models.TextField(default={})
    SECRETS = models.TextField(default=[])
    TRACKERS = models.TextField(default={})


class StaticAnalyzerWindows(models.Model):
    class Meta:
        """Meta class for StaticAnalyzerWindows model."""

        permissions = (P.DELETE.value, P.SCAN.value)

    FILE_NAME = models.CharField(max_length=260, default='')
    APP_NAME = models.CharField(max_length=260, default='')
    PUBLISHER_NAME = models.TextField(default='')
    SIZE = models.CharField(max_length=50, default='')
    MD5 = models.CharField(max_length=32, default='', primary_key=True)
    SHA1 = models.CharField(max_length=40, default='')
    SHA256 = models.CharField(max_length=64, default='')
    APP_VERSION = models.TextField(default='')
    ARCHITECTURE = models.TextField(default='')
    COMPILER_VERSION = models.TextField(default='')
    VISUAL_STUDIO_VERSION = models.TextField(default='')
    VISUAL_STUDIO_EDITION = models.TextField(default='')
    TARGET_OS = models.TextField(default='')
    APPX_DLL_VERSION = models.TextField(default='')
    PROJ_GUID = models.TextField(default='')
    OPTI_TOOL = models.TextField(default='')
    TARGET_RUN = models.TextField(default='')
    FILES = models.TextField(default=[])
    STRINGS = models.TextField(default=[])
    BINARY_ANALYSIS = models.TextField(default=[])
    BINARY_WARNINGS = models.TextField(default=[])


class SuppressFindings(models.Model):
    class Meta:
        """Meta class for SuppressFindings model."""

        permissions = (P.DELETE.value, P.SCAN.value, P.SUPPRESS.value)

    PACKAGE_NAME = models.CharField(max_length=260, default='')
    SUPPRESS_RULE_ID = models.TextField(default=[])
    SUPPRESS_FILES = models.TextField(default={})
    SUPPRESS_TYPE = models.TextField(default='')


class EnqueuedTask(models.Model):
    task_id = models.CharField(max_length=255)
    checksum = models.CharField(max_length=255)
    file_name = models.CharField(max_length=255)
    created_at = models.DateTimeField(default=timezone.now)
    status = models.CharField(max_length=255, default='Enqueued')
    started_at = models.DateTimeField(null=True)
    completed_at = models.DateTimeField(null=True)
    app_name = models.CharField(max_length=255, default='')

    def __str__(self):
        return f'{self.name} ({self.status})'
